<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

defined('_JEXEC') or die;

JLoader::register('MobileassistantconnectorHelper', JPATH_ADMINISTRATOR . '/components/com_mobileassistantconnector/helpers/mobileassistantconnector.php');

class MobileassistantconnectorControllerUser extends JControllerForm
{
     /**
     * Method override to check if you can add a new record.
     *
     * @param   array  $data  An array of input data.
     *
     * @return  boolean
     *
     * @since   1.6
     */
    protected function allowAdd($data = array())
    {
        // If the category has been passed in the URL check it.
        $allow = JFactory::getUser()->authorise('user.edit', $this->option);

        if ($allow !== null)
        {
            return $allow;
        }

        // In the absence of better information, revert to the component permissions.
        return parent::allowAdd($data);
    }

    /**
     * Method to check if you can edit a record.
     *
     * @param   array   $data  An array of input data.
     * @param   string  $key   The name of the key for the primary key.
     *
     * @return  boolean
     *
     * @since   1.6
     */
    protected function allowEdit($data = array(), $key = 'id')
    {
        // Since there is no asset tracking, revert to the component permissions.
        return parent::allowEdit($data, $key);
    }

    /**
     * Method to run batch operations.
     *
     * @param   object  $model  The model.
     *
     * @return  boolean   True if successful, false otherwise and internal error is set.
     *
     * @since   2.5
     */
    public function batch($model = null)
    {
        JSession::checkToken() or jexit(JText::_('JINVALID_TOKEN'));

        // Set the model
        $model = $this->getModel('User', '', array());

        // Preset the redirect
        $this->setRedirect(JRoute::_('index.php?option=com_mobileassistantconnector&view=user' . $this->getRedirectToListAppend(), false));

        return parent::batch($model);
    }

    private function changeUserStatus($status)
    {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $userIds = JRequest::getVar('user_id');
        $count = count($userIds);

        for ($i = 0; $i < $count; $i++) {
            $query
                ->update($db->quoteName('#__mobassistantconnector_users'))
                ->set($db->quoteName('status') . ' = ' . (int)$status)
                ->where($db->quoteName('user_id') . ' = ' . $userIds[$i]);
            $db->setQuery($query);
            $db->execute();
        }

        $text = $status == 1
            ? JFactory::getApplication()->enqueueMessage('%s user(s) have been enabled.', count($userIds))
            : JFactory::getApplication()->enqueueMessage('%s user(s) have been disabled.', count($userIds));

        echo $text;
        $app = JFactory::getApplication();
        $url = JRoute::_('index.php?option=com_mobileassistantconnector&view=user&layout=edit&user_id=' . JRequest::getVar('user_id') , false);
        $app->redirect($url);
    }

    /**
     * Mass delete selected push records
     */
    public function massDeleteDeviceAction()
    {
        $this->deleteDevice(JRequest::getVar('cid'));
    }

    private function changeStatusAccount($pushIds, $value)
    {
        $db = JFactory::getDbo();
//        $account_result = array();

        $account_sql = $db->getQuery(true);
        $account_sql
            ->select($db->quoteName('a.id'))
            ->from($db->quoteName('#__mobassistantconnector_accounts', 'a'))
            ->leftJoin('#__mobassistantconnector_devices AS d ON d.account_id = a.id')
            ->leftJoin('#__mobassistantconnector_push_notifications AS pn ON pn.device_unique_id = d.device_unique_id')
            ->where('pn.id IN (' . implode(',', $pushIds) . ')');
        $db->setQuery($account_sql);
        $account_result = $db->loadColumn();

        $update_status = $db->getQuery(true);
        $update_status
            ->update($db->quoteName('#__mobassistantconnector_accounts', 'a'))
            ->set('a.status = ' . $value)
            ->where('a.id IN (' . implode(',', $account_result) . ')');
        $db->setQuery($update_status);
        $db->execute();

        $count_account = count($pushIds);
//
        JFactory::getApplication()->enqueueMessage('A total of ' . $count_account . ' account(s) have been updated.');

        $this->editAction();
    }

    public function editAction()
    {
        $db = JFactory::getDbo();

        // 1. Get ID and create model
        $id = JRequest::getVar('user_id');
        $sql = $db->getQuery(true);
        $sql
            ->select('*')
            ->from($db->quoteName('#__mobassistantconnector_users', 'u'))
            ->where($db->quoteName('u.user_id') . ' = ' . $id);
        $db->setQuery($sql);
        $model = $db->loadAssocList();

        // 2. Initial checking
        if ($id) {
            if (!$model->id) {
                $app = JFactory::getApplication();
                $url = JRoute::_('index.php?option=com_mobileassistantconnector&view=user&layout=edit&user_id=' . JRequest::getVar('user_id'), false);
                $app->redirect($url);
                return;
            }
        }
    }

    private function deleteDevice($pushIds)
    {
        $db = JFactory::getDbo();
        if (empty($pushIds)) {
            JFactory::getApplication()->enqueueMessage('Nothing selected');
        } else {
            $db->setQuery('DELETE pn.* FROM #__mobassistantconnector_push_notifications AS pn 
                WHERE pn.id IN (' . implode(',', $pushIds) . ')' );
            $db->execute();

            MobileassistantconnectorHelperDevice_and_push_notifications::deleteEmptyDevices();
            MobileassistantconnectorHelperDevice_and_push_notifications::deleteEmptyAccounts();

            try {
                JFactory::getApplication()->enqueueMessage('A total of ' . count($pushIds) . ' record(s) have been deleted.');
            } catch (Exception $e) {
                JFactory::getApplication()->enqueueMessage($e->getMessage(), 'error');
            }
        }
        $this->editAction();
    }

    /**
     * Enable selected users
     */
    public function massEnableAction()
    {
        $this->changeUserStatus(1);
    }

    /**
     * Disable selected users
     */
    public function massDisableAction()
    {
        $this->changeUserStatus(0);
    }

    /**
     * Enable selected accounts
     */
    public function massEnableAccountAction()
    {
        $this->massChangeStatusAccountAction(1);
    }

    /**
     * Disable selected accounts
     */
    public function massDisableAccountAction()
    {
        $this->massChangeStatusAccountAction(0);
    }

    /**
     * Mass change status of selected accounts
     */
    public function massChangeStatusAccountAction($value)
    {
        $this->changeStatusAccount(JRequest::getVar('cid'), $value);

    }

    /**
     * Enable account
     */
    public function enableAccountAction()
    {
        $this->changeStatusAccountAction(1);
    }

    /**
     * Disable selected accounts
     */
    public function disableAccountAction()
    {
        $this->changeStatusAccountAction(0);
    }

    /**
     * Change status of selected account
     */
    public function changeStatusAccountAction($value)
    {
        $this->changeStatusAccount(JRequest::getVar('cid'), $value);
    }

    /**
     * Delete push record
     */
    public function deleteDeviceAction()
    {
        $this->deleteDevice(JRequest::getVar('cid'));
    }
}
