<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

class MobileassistantconnectorHelperDevice_and_push_notifications extends JHelperContent
{
    public static function deleteEmptyDevices()
    {
        $db = JFactory::getDbo();

        try {
            $sql = $db->getQuery(true);
            $sql
                ->select('d.device_unique_id')
                ->from($db->quoteName('#__mobassistantconnector_devices', 'd'))
                ->leftJoin('#__mobassistantconnector_push_notifications AS pn ON pn.device_unique_id = d.device_unique_id')
                ->where('pn.device_unique_id IS NULL');
            $db->setQuery($sql);
            $device_id = implode(',', $db->loadColumn());

            $devices = $db->getQuery(true);
            $devices
                ->delete($db->quoteName('#__mobassistantconnector_devices'))
                ->where('device_unique_id IN(' . $device_id . ')');
            $db->setQuery($devices);
            $db->execute();

            $result = true;
        } catch (Exception $e) {
            $result = false;
            JLog::add(JText::_("Unable to delete device ({$e->getMessage()})."), JLog::ERROR, 'com_mobileassistantconnector');
        }

        return $result;
    }

    public static function deleteEmptyAccounts()
    {
        $db = JFactory::getDbo();
        $sql = $db->getQuery(true);

        try {
            $sql
                ->select('a.id')
                ->from($db->quoteName('#__mobassistantconnector_accounts', 'a'))
                ->leftJoin('#__mobassistantconnector_devices AS d ON d.account_id = a.id')
                ->where('d.account_id IS NULL');
            $db->setQuery($sql);
            $account_id = implode(',', $db->loadColumn());

            $accounts = $db->getQuery(true);
            $accounts
                ->delete($db->quoteName('#__mobassistantconnector_accounts'))
                ->where('id IN(' . $account_id . ')');
            $db->setQuery($accounts);
            $db->execute();
            $result = true;
        } catch (Exception $e) {
            $result = false;
            JLog::add(JText::_("Unable to delete account ({$e->getMessage()})."), JLog::ERROR, 'com_mobileassistantconnector');
        }

        return $result;
    }

    public static function addDevice($data)
    {
        $result = false;
        $db = JFactory::getDbo();
        $sql = $db->getQuery(true);

        if (!empty($data)) {
            try {
                $sql
                    ->insert($db->quoteName('#__mobassistantconnector_devices'))
                    ->set('device_unique = ' . $data['device_unique'])
                    ->set('account_id = ' . $data['account_id'])
                    ->set('device_name = ' . $data['device_name'])
                    ->set('last_activity = ' . $data['last_activity']);
                $db->setQuery($sql);
                $db->execute();
            } catch (Exception $e) {
                JLog::add(JText::_("Unable to insert device ({$e->getMessage()})."), JLog::ERROR, 'com_mobileassistantconnector');
            }
        }

        return $result;
    }

    public static function addPushNotification($data)
    {
        $result = true;
        $db = JFactory::getDbo();
        $sql = $db->getQuery(true);

        try {
            $sql
                ->insert($db->quoteName('#__mobassistantconnector_push_notifications'))
                ->set('device_id = ' . $data['device_unique_id'])
                ->set('device_unique_id = ' . $data['device_unique_id'])
                ->set('user_id = ' . $data['user_id'])
                ->set('store_group_id = ' . $data['store_group_id'])
                ->set('currency_code = ' . $data['currency_code']);

            if (isset($data['new_order'])) {
                $sql->set('new_order = '. $data['new_order']);
            }

            if (isset($data['new_customer'])) {
                $sql->set('new_customer = ' . $data['new_customer']);
            }

            if (isset($data['order_statuses'])) {
                $sql->set('order_statuses = ' . $data['order_statuses']);
            }

            $db->setQuery($sql);
            $db->execute();

        } catch (Exception $e) {
            $result = false;
            JLog::add(JText::_("Unable to add push notification ({$e->getMessage()})."), JLog::ERROR, 'com_mobileassistantconnector');
        }

        return $result;
    }
}