<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

class MobileassistantconnectorHelperPermissions extends JHelperContent
{
    public static $extension = 'com_mobileassistantconnector';

    public static function getAllowedActions($user_id)
    {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query
            ->select($db->quoteName('allowed_actions'))
            ->from($db->quoteName('#__mobassistantconnector_users'))
            ->where('user_id = ' . $user_id);
        $db->setQuery($query);
        $user_actions = $db->loadResult();

        if ($user_actions === false) {
            return 'error';
        }

        return self::getActionsByCodes($user_actions);
    }
    
    public static function getRestrictedActions()
    {
        // Visible for users in back-end
        $restricted_actions = array(
            array(
                'group_name' => 'Push notification settings',
                'child' => array(
                    array(
                        'code' => 'push_notification_settings_new_order',
                        'name' => 'New order',
                    ),
                    array(
                        'code' => 'push_notification_settings_new_customer',
                        'name' => 'New customer',
                    ),
                    /*array(
                        'code' => 'push_notification_settings_order_statuses',
                        'name' => 'Order statuses',
                    ),*/
                ),
            ),
            array(
                'group_name' => 'Store statistics',
                'child' => array(
                    array(
                        'code' => 'store_stats',
                        'name' => 'Store statistics',
                    ),
                ),
            ),
            array(
                'group_name' => 'Products',
                'child' => array(
                    array(
                        'code' => 'products_list',
                        'name' => 'Product list',
                    ),
                    array(
                        'code' => 'product_details',
                        'name' => 'Product details',
                    ),
                ),
            ),
            array(
                'group_name' => 'Customers',
                'child' => array(
                    array(
                        'code' => 'customers_list',
                        'name' => 'Customer list',
                    ),
                    array(
                        'code' => 'customer_details',
                        'name' => 'Customer details',
                    ),
                ),
            ),
            array(
                'group_name' => 'Orders',
                'child' => array(
                    array(
                        'code' => 'orders_list',
                        'name' => 'Order list',
                    ),
                    array(
                        'code' => 'order_details',
                        'name' => 'Order details',
                    ),
                    array(
                        'code' => 'update_order_status',
                        'name' => 'Order status updating',
                    ),
                ),
            ),
            array(
                'group_name' => 'Abandoned carts',
                'child' => array(
                    array(
                        'code' => 'abandoned_carts_list',
                        'name' => 'Abandoned cart list',
                    ),
                    array(
                        'code' => 'abandoned_cart_details',
                        'name' => 'Abandoned cart details',
                    ),
                ),
            ),
        );

        return $restricted_actions;
    }

    public static function getUserAllowedActionsAsString($action_codes, $restricted_actions)
    {
        $is_allowed_all = true;
        $result = '';
        $groups_count = count($restricted_actions);

        for ($i = 0; $i < $groups_count; $i++) {
            $child_count = count($restricted_actions[$i]['child']);
            $tmp = '';

            for ($j = 0; $j < $child_count; $j++) {
                if (in_array($restricted_actions[$i]['child'][$j]['code'], $action_codes)) {
                    if (!empty($tmp)) {
                        $tmp .= ', ';
                    }

                    $tmp .= $restricted_actions[$i]['child'][$j]['name'];
                } else {
                    $is_allowed_all = false;
                }
            }

            if (!empty($tmp)) {
                if (!empty($result)) {
                    $result .= ', ';
                }

                $result .= $restricted_actions[$i]['group_name'].' ('.$tmp.')';
            }
        }

        if ($is_allowed_all) {
            $result = 'All';
        }

        return $result;
    }

    private static function getActionsByCodes($actions_codes)
    {
        $restricted_actions = self::getRestrictedActions();
        $actions = array();

        if (!empty($actions_codes)) {
            $actions = explode(';', $actions_codes);
        }

        return self::getActionsAsArray($restricted_actions, $actions);
    }

    private static function getActionsAsArray($restricted_actions, $actions_codes)
    {
        $groups_count = count($restricted_actions);

        for ($i = 0; $i < $groups_count; $i++) {
            $child_count = count($restricted_actions[$i]['child']);

            for ($j = 0; $j < $child_count; $j++) {
                if (!empty($actions_codes) && in_array($restricted_actions[$i]['child'][$j]['code'], $actions_codes)) {
                    $restricted_actions[$i]['child'][$j]['allowed'] = 1;
                } else {
                    $restricted_actions[$i]['child'][$j]['allowed'] = 0;
                }
            }
        }

        return $restricted_actions;
    }
}