<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

defined('_JEXEC') or die('Restricted access');

defined('VMPATH_ROOT') or define('VMPATH_ROOT', JPATH_ROOT);
defined('VMPATH_ADMIN') or define('VMPATH_ADMIN', VMPATH_ROOT . '/administrator/components/com_virtuemart');

class com_mobileassistantconnectorInstallerScript
{
    /**
     * method to install the component
     *
     * @return void
     */
    function install()
    {
        $_REQUEST['install'] = 1;
        if (!class_exists('JFile')) require(VMPATH_LIBS . '/joomla/filesystem/file.php');
        if (!class_exists('JFolder')) require(VMPATH_LIBS . '/joomla/filesystem/folder.php');

        $this->checkAddDefaultUser();
    }

    private function checkAddDefaultUser()
    {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);

        $profile = new stdClass();
        $profile->username = '1';
        $profile->password = md5('1');
        $profile->allowed_actions = implode(';', $this->getActionsCodes());
        $profile->qr_code_hash = hash('sha256', time());
        $profile->status = 1;

        $result = JFactory::getDbo()->insertObject('#__mobassistantconnector_users', $profile);
    }

    /**
     * method to update the component
     *
     * @return void
     */
    function update($parent)
    {
        // $parent is the class calling this method
        echo '<p>' . JText::sprintf('COM_MOBILEASSISTANTCONNECTOR_UPDATE_TEXT', $parent->get('manifest')->version) . '</p>';
    }

    private function getActionsCodes()
    {
        $actions = $this->getRestrictedActions();
        $result = array();
        $groups_count = count($actions);

        for ($i = 0; $i < $groups_count; $i++) {
            $child_count = count($actions[$i]['child']);

            for ($j = 0; $j < $child_count; $j++) {
                $result[] = $actions[$i]['child'][$j]['code'];
            }
        }
        return $result;

    }

    public function getRestrictedActions()
    {
        // Visible for users in back-end
        $restricted_actions = array(
            array(
                'group_name' => 'Push notification settings',
                'child'      => array(
                    array(
                        'code' => 'push_notification_settings_new_order',
                        'name' => 'New order',
                    ),
                    array(
                        'code' => 'push_notification_settings_new_customer',
                        'name' => 'New customer',
                    ),
                    array(
                        'code' => 'push_notification_settings_order_statuses',
                        'name' => 'Order statuses',
                    ),
                ),
            ),
            array(
                'group_name' => 'Store statistics',
                'child'      => array(
                    array(
                        'code' => 'store_stats',
                        'name' => 'Store statistics',
                    ),
                ),
            ),
            array(
                'group_name' => 'Products',
                'child'      => array(
                    array(
                        'code' => 'products_list',
                        'name' => 'Product list',
                    ),
                    array(
                        'code' => 'product_details',
                        'name' => 'Product details',
                    ),
                ),
            ),
            array(
                'group_name' => 'Customers',
                'child'      => array(
                    array(
                        'code' => 'customers_list',
                        'name' => 'Customer list',
                    ),
                    array(
                        'code' => 'customer_details',
                        'name' => 'Customer details',
                    ),
                ),
            ),
            array(
                'group_name' => 'Orders',
                'child'      => array(
                    array(
                        'code' => 'orders_list',
                        'name' => 'Order list',
                    ),
                    array(
                        'code' => 'order_details',
                        'name' => 'Order details',
                    ),
                    array(
                        'code' => 'update_order_status',
                        'name' => 'Order status updating',
                    ),
                    array(
                        'code' => 'update_order_tracking_number',
                        'name' => 'Order tracking number updating',
                    ),
                ),
            ),
            array(
                'group_name' => 'Abandoned carts',
                'child'      => array(
                    array(
                        'code' => 'abandoned_carts_list',
                        'name' => 'Abandoned cart list',
                    ),
                    array(
                        'code' => 'abandoned_cart_details',
                        'name' => 'Abandoned cart details',
                    ),
                ),
            ),
        );

        return $restricted_actions;
    }

    public function getRestrictedActionsToFunctions()
    {
        $restricted_actions_to_functions = array(
            'push_notification_settings_new_order' => array(
                'push_notification_settings',
                'delete_push_config',
            ),
            'push_notification_settings_new_customer' => array(
                'push_notification_settings',
                'delete_push_config',
            ),
            'push_notification_settings_order_statuses' => array(
                'push_notification_settings',
                'delete_push_config',
            ),
            'store_stats' => array(
                'get_store_stats',
                'get_data_graphs',
                'get_status_stats',
            ),
            'products_list' => array(
                'search_products',
                'search_products_ordered',
            ),
            'product_details' => array(
                'get_products_info',
                'get_products_descr',
            ),
            'customers_list' => array(
                'get_customers',
            ),
            'customer_details' => array(
                'get_customers_info',
            ),
            'orders_list' => array(
                'get_orders',
            ),
            'order_details' => array(
                'get_orders_info',
                'get_order_pdf',
            ),
            'update_order_status' => array(
                'set_order_action',
                'change_status',
            ),
            'update_order_tracking_number' => array(
                'set_order_action',
                'update_track_number',
            ),
            'abandoned_carts_list' => array(
                'get_abandoned_carts_list',
            ),
            'abandoned_cart_details' => array(
                'get_abandoned_cart_details',
            ),
        );

        return $restricted_actions_to_functions;
    }

    public function getAlwaysAllowedFunctions()
    {
        $allowed_functions_always = array(
            'run_self_test',
            'get_stores',
            'get_currencies',
            'get_store_title',
            'get_orders_statuses',
            'get_carriers',
        );

        return $allowed_functions_always;
    }
}

