<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

defined('_JEXEC') or die;

use Joomla\Registry\Registry;

class MobileassistantconnectorModelMobileassistantconnector extends JModelList
{

    protected $_extension = 'com_mobileassistantconnector';

    public function __construct($config = array())
    {
        if (empty($config['filter_fields'])) {
            $config['filter_fields'] = array(
                'user_id', 'a.user_id',
                'username', 'a.username',
                'password', 'a.password',
                'status', 'a.status',
                'allowed_actions', 'a.allowed_actions',
            );

            $assoc = JLanguageAssociations::isEnabled();
            if ($assoc) {
                $config['filter_fields'][] = 'association';
            }
        }

        parent::__construct($config);
    }

    protected function getStoreId($id = '')
    {
        // Compile the store id.
        $id .= ':' . $this->getState('filter.search');
        $id .= ':' . $this->getState('filter.published');
        $id .= ':' . $this->getState('filter.category_id');
        $id .= ':' . $this->getState('filter.access');
        $id .= ':' . $this->getState('filter.language');
        $id .= ':' . $this->getState('filter.tag');
        $id .= ':' . $this->getState('filter.level');

        return parent::getStoreId($id);
    }

    public function getItems()
    {
        // Invoke the parent getItems method to get the main list
        $items = parent::getItems();

        // Convert the params field into an object, saving original in _params
        foreach ($items as $item)
        {
            if (!isset($this->_params))
            {
                $params = new Registry;
                $item->params = $params;
                $params->loadString($item->params);
            }
        }

        return $items;
    }

    protected function getListQuery()
    {
        // Create a new query object.
        $db = $this->getDbo();
        $query = $db->getQuery(true);

        // Select the required fields from the table.
        $query->select(
            $this->getState(
                'list.select',
                'a.user_id, a.username, a.password, a.qr_code_hash, a.allowed_actions, a.status'
            )
        );
        $query->from($db->quoteName('#__mobassistantconnector_users', 'a'));

        // Join over the associations.
        $assoc = JLanguageAssociations::isEnabled();

        if ($assoc)
        {
            $query->group('a.user_id, a.username, a.password, a.qr_code_hash, a.allowed_actions, a.status');
        }

        // Add the list ordering clause.
        $orderCol = $this->state->get('list.ordering', 'a.username');
        $orderDirn = $this->state->get('list.direction', 'ASC');

        if ($orderCol == 'a.ordering' || $orderCol == 'category_title')
        {
            $orderCol = 'c.title ' . $orderDirn . ', a.ordering';
        }

        $query->order($db->escape($orderCol . ' ' . $orderDirn));

        return $query;
    }

    public function getForm($data = array(), $loadData = true)
    {
        // Get the form.
        $form = $this->loadForm(
            'com_mobileassistantconnector.user',
            'user',
            array('control' => 'jform', 'load_data' => $loadData)
        );

        if (empty($form))
        {
            return false;
        }

        // Determine correct permissions to check.
        if ($this->getState('user.user_id'))
        {
            // Existing record. Can only edit in selected categories.
            $form->setFieldAttribute('user_id', 'action', 'core.edit');
        }
        else
        {
            // New record. Can only create in selected categories.
            $form->setFieldAttribute('user_id', 'action', 'core.create');
        }

        // Modify the form based on access controls.
            // Disable fields for display.
            $form->setFieldAttribute('status', 'disabled', 'true');
            $form->setFieldAttribute('user_id', 'disabled', 'true');
            $form->setFieldAttribute('username', 'disabled', 'true');
            $form->setFieldAttribute('allowed_actions', 'disabled', 'true');
            $form->setFieldAttribute('qr_code_hash', 'disabled', 'true');
            $form->setFieldAttribute('password', 'disabled', 'true');

            // Disable fields while saving.
            // The controller has already verified this is a record you can edit.
            $form->setFieldAttribute('status', 'filter', 'unset');
            $form->setFieldAttribute('user_id', 'filter', 'unset');
            $form->setFieldAttribute('username', 'filter', 'unset');
            $form->setFieldAttribute('allowed_actions', 'filter', 'unset');
            $form->setFieldAttribute('qr_code_hash', 'filter', 'unset');
            $form->setFieldAttribute('password', 'filter', 'unset');

        return $form;
    }
}
