<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

defined('_JEXEC') or die;

use Joomla\Registry\Registry;

JLoader::register('MobileassistantconnectorHelper', JPATH_ADMINISTRATOR . '/components/com_mobileassistantconnector/helpers/mobileassistantconnector.php');

class MobileassistantconnectorModelUser extends JModelAdmin
{
    public $typeAlias = 'com_mobileassistantconnector.user';

    protected $associationsContext = 'com_mobileassistantconnector.user';

    protected $text_prefix = 'COM_MOBILEASSISTANTCONNECTOR';

    /**
     * Method to test whether a record can be deleted.
     *
     * @param   object  $record  A record object.
     *
     * @return  boolean  True if allowed to delete the record. Defaults to the permission set in the component.
     *
     * @since   1.6
     */
    protected function canDelete($record)
    {
        $user = JFactory::getUser();

        return $user->authorise('core.delete', $this->option);
    }

    /**
     * Method to test whether a record can have its state changed.
     *
     * @param   object  $record  A record object.
     *
     * @return  boolean  True if allowed to change the state of the record. Defaults to the permission set in the component.
     *
     * @since   1.6
     */
    protected function canEditState($record)
    {
        $user = JFactory::getUser();

        if (!empty($record->user_id))
        {
            return $user->authorise('user.edit', 'com_mobileassistantconnector.user.' . (int) $record->user_id);
        }
        else
        {
            return parent::canEditState($record);
        }
    }

    public function getTable($type = 'User', $prefix = 'MobileassistantconnectorTable', $config = array())
    {
        return JTable::getInstance($type, $prefix, $config);
    }

    public function getForm($data = array(), $loadData = true)
    {
        // Get the form.
        $form = $this->loadForm('com_mobileassistantconnector.user', 'user', array('control' => 'jform', 'load_data' => $loadData));

        if (empty($form))
        {
            return false;
        }

        return $form;
    }

    protected function loadFormData()
    {
        $data = $this->getItem(JRequest::getString('id'));

        $this->preprocessData('com_mobileassistantconnector.user', $data);

        return $data;
    }

    /**
     * Method to save the form data.
     *
     * @param   array  $data  The form data.
     *
     * @return  boolean  True on success.
     *
     * @since   3.0
     */
    public function save($data, $trigger=true)
    {
        $data['allowed_actions'] = implode(';', JFactory::getApplication()->input->get('allowed_actions', array(), 'array'));
        $form_data = JRequest::getVar('jform');
        $db = JFactory::getDbo();
        $sql = $db->getQuery(true);

        $sql
            ->select('u.username, u.user_id')
            ->from($db->quoteName('#__mobassistantconnector_users', 'u'))
            ->where('u.username = "' . $form_data['username'] . '"');

        if (JRequest::getVar('user_id') > 0) {
            $sql->where('u.user_id != ' . (int)JRequest::getVar('user_id'));
        }

        $db->setQuery($sql);

        if (empty($db->loadAssocList())) {
            if (JRequest::getVar('user_id') > 0) {
                $query_hash = $db->getQuery(true);
                $query_hash
                    ->select('u.username, u.user_id, u.password')
                    ->from($db->quoteName('#__mobassistantconnector_users', 'u'))
                    ->where('u.user_id = ' . (int)JRequest::getVar('user_id'));
                $db->setQuery($query_hash);
                $user = $db->loadObject();

                if ($user->username != $form_data['username'] || $user->password != $form_data['password']) {
                    $data['qr_code_hash'] = hash('sha256', microtime() . rand());

                    if ($user->password != $form_data['password']) {
                        $data['password'] = md5($form_data['password']);
                    }
                }
            } else {
                $data['password'] = md5($form_data['password']);
                $data['qr_code_hash'] = hash('sha256', microtime() . rand());
            }

            return parent::save($data);

        } else {
            $this->setError(JText::sprintf('Another user exists with the same username', $this->getError()));
            return false;
        }

    }

    /**
     * Method to get a single record.
     *
     * @param   integer  $pk  The id of the primary key.
     *
     * @return  mixed  Object on success, false on failure.
     *
     * @since   1.6
     */
    public function getItem($pk = null)
    {
        $item = parent::getItem(JRequest::getString('id'));

        return $item;
    }
}
