<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

defined('_JEXEC') or die;

use Joomla\Registry\Registry;
jimport('joomla.application.component.modellist');

class MobileassistantconnectorModelUsers extends JModelList
{

    protected $_extension = 'com_mobileassistantconnector';

    public function __construct($config = array())
    {
        if (empty($config['filter_fields'])) {
            $config['filter_fields'] = array(
                'user_id', 'a.user_id',
                'username', 'a.username',
                'password', 'a.password',
                'status', 'a.status',
                'allowed_actions', 'a.allowed_actions',
            );

            if (version_compare(JVERSION,'3.0', '>=')) {
                $assoc = JLanguageAssociations::isEnabled();
                if ($assoc) {
                    $config['filter_fields'][] = 'association';
                }
            }
        }

        parent::__construct($config);
    }

    public function getItems()
    {
        // Invoke the parent getItems method to get the main list
        $items = parent::getItems();

        // Convert the params field into an object, saving original in _params
        foreach ($items as $item)
        {
            if (!isset($this->_params))
            {
                $params = new Registry;
                $item->params = $params;
                $params->loadString($item->params);
            }
        }

        return $items;
    }

    protected function getListQuery()
    {
        // Create a new query object.
        $db = $this->getDbo();
        $query = $db->getQuery(true);

        // Select the required fields from the table.
        $query->select(
            $this->getState(
                'list.select',
                'a.user_id, a.username, a.password, a.qr_code_hash, a.allowed_actions, a.status'
            )
        );
        $query->from($db->quoteName('#__mobassistantconnector_users', 'a'));

        // Join over the associations.
        if (version_compare(JVERSION,'3.0', '>=')) {
            $assoc = JLanguageAssociations::isEnabled();

            if ($assoc) {
                $query->group('a.user_id, a.username, a.password, a.qr_code_hash, a.allowed_actions, a.status');
            }
        }

        $published = $this->getState('filter.published');

        if (is_numeric($published))
        {
            $query->where($db->quoteName('a.state') . ' = ' . (int) $published);
        }
        elseif ($published === '')
        {
            $query->where($db->quoteName('a.state') . ' IN (0, 1)');
        }

        // Add the list ordering clause.
        $orderCol = $this->state->get('list.ordering', 'a.username');
        $orderDirn = $this->state->get('list.direction', 'ASC');

        $query->order($db->escape($orderCol . ' ' . $orderDirn));

        return $query;
    }

    /**
     * Returns a reference to the a Table object, always creating it.
     *
     * @param   string  $type    The table type to instantiate
     * @param   string  $prefix  A prefix for the table class name. Optional.
     * @param   array   $config  Configuration array for model. Optional.
     *
     * @return  JTable  A JTable object
     *
     * @since   1.6
     */
    public function getTable($type = 'User', $prefix = 'MobileassistantconnectorTable', $config = array())
    {
        return JTable::getInstance($type, $prefix, $config);
    }
}
