<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

defined('_JEXEC') or die;

/**
 * Mobileassistantconnector Table class.
 *
 * @since  1.6
 */
class MobileassistantconnectorTableUser extends JTable
{
    /**
     * Constructor
     *
     * @param   JDatabaseDriver  &$db  A database connector object
     */
    public function __construct(&$db)
    {
        parent::__construct('#__mobassistantconnector_users', 'user_id', $db);

        JTableObserverTags::createObserver($this, array('typeAlias' => 'com_mobileassistantconnector.user'));
        JTableObserverContenthistory::createObserver($this, array('typeAlias' => 'com_mobileassistantconnector.user'));

        if (JVERSION >= "3.4.0") {
            $this->setColumnAlias('published', 'status');
        }
    }

    /**
     * Overloaded check method to ensure data integrity.
     *
     * @return  boolean  True on success.
     */
    public function check()
    {
        // Check for valid name.
        if (trim($this->username) == '')
        {
            $this->setError(JText::_('Please provide a valid name.'));
            return false;
        }
        return true;
    }

    /**
     * Overriden JTable::store to set modified data.
     *
     * @param   boolean  $updateNulls  True to update fields even if they are null.
     *
     * @return  boolean  True on success.
     *
     * @since   1.6
     */
    public function store($updateNulls = false)
    {
        $user = JFactory::getUser();

        // Verify that the alias is unique
        $table = JTable::getInstance('User', 'MobileassistantconnectorTable');

        if ($table->load(array('user_id' => $this->user_id, 'username' => $this->username)) && ($table->user_id != $this->user_id || $this->user_id == 0))
        {
            $this->setError(JText::_('COM_MOB_ASSISTANT_CONNECTOR_ERROR_UNIQUE_ALIAS'));

            return false;
        }

        return parent::store($updateNulls);
    }

    function delete( $id=null , $where = 0 ){

        $db = JFactory::getDbo();

        $user = "DELETE u.*, sk.*, pn.* FROM `#__mobassistantconnector_users` AS u
                LEFT JOIN `#__mobassistantconnector_session_keys` AS sk ON sk.user_id = u.user_id
                LEFT JOIN `#__mobassistantconnector_push_notifications` AS pn ON pn.user_id = u.user_id
                WHERE  u.user_id = " . $id;
        $db->setQuery($user);
        
        if($db->execute() === false) {
            return false;
        }

        MobileassistantconnectorHelperDevice_and_push_notifications::deleteEmptyDevices();
        MobileassistantconnectorHelperDevice_and_push_notifications::deleteEmptyAccounts();

        return true;
    }
}
