<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

defined('_JEXEC') or die;
JHtml::_('jquery.framework');

require_once JPATH_COMPONENT_ADMINISTRATOR.'/helpers/qrcode.php';

$document = JFactory::getDocument();
$document->addStyleSheet(JURI::base(). "components/com_mobileassistantconnector/assets/css/style.css");
$document->addScript(JURI::base(). "components/com_mobileassistantconnector/assets/js/user_edit.js");
$document->addScript(JURI::root(). "components/com_mobileassistantconnector/assets/js/qrcode.min.js");
$document->addScript(JURI::base(). "components/com_mobileassistantconnector/assets/js/qrcode_app_user.js");
$document->addScript(JUri::base(). "components/com_mobileassistantconnector/assets/js/check_permissions.js");

if (JVERSION >= '3.4.0'){
    JHtml::_('behavior.formvalidator');
}
JHtml::_('behavior.keepalive');
JHtml::_('formbehavior.chosen', 'select');

$user      = JFactory::getUser();
$canEdit = $user->authorise('user.edit','com_mobileassistantconnector');
$canView = $user->authorise('user.view','com_mobileassistantconnector');

$db = JFactory::getDBO();
$query = $db->getQuery(true);

if(JRequest::getVar('user_id')) {
    $query
        ->select('
        d.device_unique_id,
        d.device_name, 
        d.last_activity, 
        a.id as id,
        a.account_email,
        a.status,
        pn.id AS id_pn,
        pn.new_order, 
        pn.new_customer,
        pn.order_statuses,
        pn.app_connection_id,
        vc.currency_code_3 AS currency_code
    ')
        ->from($db->quoteName('#__mobassistantconnector_devices', 'd'))
        ->leftJoin('#__mobassistantconnector_accounts AS a ON(a.id = d.account_id)')
        ->leftJoin('#__mobassistantconnector_push_notifications AS pn ON (d.device_unique_id = pn.device_unique_id)')
        ->leftJoin('#__virtuemart_currencies AS vc ON (pn.currency_code = vc.virtuemart_currency_id)')
        ->where('pn.user_id = ' . JRequest::getVar('user_id'))
        ->order($db->quoteName('d.device_unique_id'));

    $db->setQuery($query);
    $push_items = $db->loadObjectList();
}

JFactory::getDocument()->addScriptDeclaration('
	Joomla.submitbutton = function(task)
	{
		if (task == "user.cancel" || document.formvalidator.isValid(document.getElementById("adminForm"))) {
			Joomla.submitform(task, document.getElementById("adminForm"));

			if (task !== "user.apply")
			{
				window.parent.jQuery("#userEdit' . $this->item->user_id . 'Modal").modal("hide");
			}
		}
	};
	
	function changeAction()
    {    
        jQuery(document).ready(function($) 
        {
            $("#select_action_chzn").change(function()
            {
                status = $("#select_action_" + this.id_pn).val();            
                $.ajax({
                    url: "index.php?option=com_mobileassistantconnector&view=user&task=user.changeStatusAccount",
                    type: "POST",
                    data: {"status": status}                
                })
            })
        });
    }
	
');
?>

<form action="<?php echo JRoute::_('index.php?option=com_mobileassistantconnector&view=user&layout=edit&user_id=' .
    (int) $this->item->user_id); ?> " method="post" name="adminForm" id="adminForm" class="form-validate">

    <?php if (!empty( $this->sidebar)) : ?>
    <div id="j-sidebar-container" class="span2">
        <?php echo $this->sidebar; ?>
    </div>
    <div id="j-main-container" class="span10">
        <?php else : ?>
        <div id="j-main-container">
            <?php endif;?>
            <?php echo JLayoutHelper::render('joomla.edit.title_alias', $this); ?>

            <div class="form-horizontal">
                <?php echo JHtml::_('bootstrap.startTabSet', 'myTab', array('active' => 'details')); ?>
                <?php echo JHtml::_('bootstrap.addTab', 'myTab', 'details', JText::_('User information')); ?>

                <div class="row-fluid form-horizontal-desktop">
                    <?php

                    if ( !$canEdit ) {
                        $this->form->setFieldAttribute('status', 'readonly', 'true', $group = null);
                        $this->form->setFieldAttribute('username', 'readonly', 'true', $group = null);
                        $this->form->setFieldAttribute('password', 'readonly', 'true', $group = null);
                    }

                    echo $this->form->renderField('status');
                    echo $this->form->renderField('username');
                    echo $this->form->renderField('password');

                    if ($this->item->user_id) {
                        ?>
                        <div class="control-group">
                            <div class="control-label">
                                <label
                                        id="jform_qr_code"
                                        class="hasPopover required"
                                        title=""
                                        data-content="Store URL and access details (login and password) for Mobile Assistant Connector
                        are encoded in this QR code. Scan it with special option available on connection settings page of
                        VirtueMart Mobile Assistant App to autofill access settings and connect to your VirtueMart store."
                                        data-original-title="QR-code">
                                    QR-code
                                </label>
                            </div>
                            <div class="controls">
                                <?php
                                echo $this->form->renderField('note_qrcode_image');
                                ?>
                            </div>
                        </div>
                        <?php
                        $hash = (string)$this->item->qr_code_hash;
                        $url = JURI::root() .
                            "index.php?option=com_mobileassistantconnector&task=mobileassistantconnector.get_qr_code&hash=$hash";

                        $this->form->setValue(
                            'qr_code_data',
                            '',
                            MobileassistantconnectorHelperQrcode::getDataToQrCode(
                                JURI::root(),
                                $this->item->username,
                                $this->item->password
                            )
                        );

                        echo $this->form->renderField('qr_code_data');

                        echo $this->form->renderField(
                            'qrcode_image',
                            'label',
                            array(
                                'label' => 'QR-code',
                                'value' => '',
                                'note' => 'Store URL and access details (login and password) for Mobile Assistant Connector are 
                        encoded in this QR code. Scan it with special option available on connection settings page of 
                        VirtueMart Mobile Assistant App to autofill access settings and connect to your Magento store.',
                                'disabled' => true
                            )
                        );
                        ?>

                        <div class="control-group">
                            <div class="control-label">
                                <label
                                        id="jform_qrcode_link"
                                        class="hasPopover required"
                                        title=""
                                        data-content="QR-code can be got by this link only if status of user is active"
                                        data-original-title="QR-code link">
                                    QR-code link
                                </label>
                            </div>
                            <div class="controls">
                                <a id="qrcode_link" href="<?= $url ?>" target="_blank">
                                    Copy link to share QR-code
                                </a>
                            </div>
                        </div>
                    <?php } ?>
                </div>
                <?php echo JHtml::_('bootstrap.endTab'); ?>

                <?php echo JHtml::_('bootstrap.addTab', 'myTab', 'misc', JText::_('Permissions')); ?>
                <div class="form-vertical">
                    <table class="form-list">
                        <div class="span">
                            <div class="mobileassistantconnector_permissions_mass_checked_class" style="text-align: right">
                                <button id="mobileassistantconnector_permissions_btn_mass_checked_1" checked_all="false">Check All</button>
                            </div>
                        </div>
                        <?php

                        //permissions tab
                        $permissions_data = MobileassistantconnectorHelperPermissions::getRestrictedActions();

                        if ($this->item->user_id) {
                            $permissions_data = MobileassistantconnectorHelperPermissions::getAllowedActions($this->item->user_id);
                        }
                        $count_data = count($permissions_data);

                        for ($i = 0; $i < $count_data; $i++) {
                        $count_child_data = count($permissions_data[$i]['child']);
                        ?>
                        <td class="label">
                            <label>
                                <?= $permissions_data[$i]['group_name'] ?>
                            </label>
                        </td>
                        <td class="value">
                            <?php
                            for ($j = 0; $j < $count_child_data; $j++) {
                                ?>
                                <ul class="cmc-checkboxes">
                                    <?php
                                    $check_option = '';
                                    $readonly = '';

                                    if ($this->item->user_id && $permissions_data[$i]['child'][$j]['allowed']) {
                                        $check_option = 'checked';
                                    }

                                    if ( !$canEdit ) {
                                        $readonly = 'readonly';
                                    }

                                    $permission_item_code = $permissions_data[$i]['child'][$j]['code'];
                                    $this->allowed_actions[] = $permissions_data[$i]['child'][$j]['name'];
                                    ?>
                                    <div class="mobileassistantconnector_user_permissions">
                                        <label for="jform_allowed_actions[]"
                                               class="mobileassistantconnector_user_permission_label">
                                            <input
                                                    type="checkbox"
                                                    id="permissions_<?= $i ?>_<?= $permissions_data[$i]['child'][$j]['code'] ?>"
                                                    name="allowed_actions[]"
                                                <?= $readonly ?>
                                                    value="<?= $permission_item_code ?>"
                                                    onclick="Joomla.isChecked(this.checked);"
                                                <?= $check_option ?> />
                                            <?= $permissions_data[$i]['child'][$j]['name'] ?>
                                        </label>
                                    </div>
                                </ul>
                                <?php
                            }
                            echo "<br></td></tr>";
                            }
                            ?>
                    </table>
                </div>
                <?php echo JHtml::_('bootstrap.endTab'); ?>
                <?php
                if ($this->item->user_id) {
                    echo JHtml::_('bootstrap.addTab', 'myTab', 'publishing', JText::_('Devices')); ?>
                    <table class="table table-striped table-hover">
                        <thead>
                        <tr>
                            <th>
                                <?php echo JHtml::_('grid.checkall'); ?>
                            </th>
                            <th>
                                <?php echo JText::_('Device Name'); ?>
                            </th>
                            <th>
                                <?php echo JText::_('Account Email'); ?>
                            </th>
                            <th>
                                <?php echo JText::_('Last Activity'); ?>
                            </th>
                            <th style="text-align: center">
                                <?php echo JText::_('App Connection ID'); ?>
                            </th>
                            <th style="text-align: center">
                                <?php echo JText::_('New Order'); ?>
                            </th>
                            <th style="text-align: center">
                                <?php echo JText::_('New Customer'); ?>
                            </th>
                            <!--<th style="text-align: center">
                                <?php /*echo JText::_('Order Statuses'); */?>
                            </th>-->
                            <th style="text-align: center">
                                <?php echo JText::_('Currency Code'); ?>
                            </th>
                            <th>
                                <?php echo JText::_('Status'); ?>
                            </th>
                        </tr>
                        </thead>
                        <tfoot>
                        <tr>
                            <td colspan="10">
                            </td>
                        </tr>
                        </tfoot>
                        <tbody>
                        <?php
                        if (!empty($push_items)) {
                            foreach ($push_items as $i => $row) {
                                ?>
                                <tr class="row<?php echo $i % 2; ?>" sortable-group-id="<?php echo $row->device_unique_id ?>">
                                    <td>
                                        <?php echo JHtml::_('grid.id', $i, $row->id_pn); ?>
                                    </td>
                                    <td align="center"><?php echo $row->device_name; ?></td>
                                    <td align="center"><?php echo $row->account_email; ?></td>
                                    <td align="center"><?php echo $row->last_activity; ?></td>
                                    <td align="center" style="text-align: center">
                                        <?php
                                        echo $row->app_connection_id;
                                        ?>
                                    </td>
                                    <td align="center">
                                        <div style="text-align: center">
                                            <?php
                                            if ($row->new_order == 1) {
                                                echo '<span class="icon-publish"></span>';
                                            } else {
                                                echo '<span class="icon-unpublish"></span>';
                                            }
                                            ?>
                                        </div>
                                    </td>
                                    <td align="center">
                                        <div style="text-align: center">
                                            <?php
                                            if ($row->new_customer == 1) {
                                                echo '<span class="icon-publish"></span>';
                                            } else {
                                                echo '<span class="icon-unpublish"></span>';
                                            }
                                            ?>
                                        </div>
                                    </td>
                                    <!--<td align="center">
                                        <div style="text-align: center">
                                            <?php
/*                                            if ($row->order_statuses == 1) {
                                                echo '<span class="icon-publish"></span>';
                                            } else {
                                                echo '<span class="icon-unpublish"></span>';
                                            }
                                            */?>
                                        </div>
                                    </td>-->
                                    <td align="center">
                                        <div style="text-align: center">
                                            <?php
                                            echo $row->currency_code;
                                            ?>
                                        </div>
                                    </td>
                                    <td>
                                        <?php
                                        if ($row->status == 1 || JRequest::getVar('select_action[]' == 1)) {
                                            echo 'Enabled';
                                        } else if ($row->status == 0 || JRequest::getVar('select_action[]' == 0)) {
                                            echo 'Disabled';
                                        }
                                        ?>
                                    </td>
                                <tr>
                                <?php
                            }
                        } else {
                            ?>
                            <tr>
                                <td colspan="11" align="center">
                                    <div style="text-align: center;">
                                        No records found
                                    </div>
                                </td>
                            </tr>
                        <?php } ?>
                        </tbody>
                    </table>
                    <?php
                    echo JHtml::_('bootstrap.endTab');
                }
                ?>
                <?php echo JHtml::_('bootstrap.endTabSet'); ?>
            </div>
            <input type="hidden" name="task" value="" />
            <input type="hidden" name="boxchecked" value="0" />
            <?php echo JHtml::_('form.token'); ?>
</form>
