<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

defined('_JEXEC') or die;
jimport('joomla.application.component.view');

if (version_compare(JVERSION,'3.0', '>=')) {
    class MobileassistantconnectorViewUserBase extends JViewLegacy {
        protected static function renderSidebar() {
            return JHtmlSidebar::render();
        }
    }
} else {
    class MobileassistantconnectorViewUserBase extends JView {
        protected static function renderSidebar() {
            return '';
        }
    }
}
class MobileassistantconnectorViewUser extends MobileassistantconnectorViewUserBase
{
    protected $item;

    protected $form;

    protected $state;

    /**
     * Execute and display a template script.
     *
     * @param   string $tpl The name of the template file to parse; automatically searches through the template paths.
     *
     * @return  mixed  A string if successful, otherwise an Error object.
     *
     * @since   1.6
     */
    public function display($tpl = null)
    {
        $this->form = $this->get('Form');
        $this->item = $this->get('Item');
        $this->state = $this->get('State');
        $this->device = $this->get('Device');

        MobileassistantconnectorHelper::addSubmenu('mobileassistantconnector');
        $this->sidebar = self::renderSidebar();

        // Check for errors.
        if (count($errors = $this->get('Errors'))) {
            JError::raiseError(500, implode("\n", $errors));

            return false;
        }

        $this->addToolbar();
        parent::display($tpl);
    }

    /**
     * Add the page title and toolbar.
     *
     * @return  void
     *
     * @since   1.6
     */
    protected function addToolbar()
    {
        JFactory::getApplication()->input->set('hidemainmenu', true);
        $canDo = JHelperContent::getActions('com_mobileassistantconnector', 'user', $this->item->user_id);

        // Since we don't track these assets at the item level, use the category id.
        JToolbarHelper::title(JText::_('Edit'));

        // If not checked out, can save the item.
        if (($canDo->get('user.edit') && $canDo->get('user.view'))) {
            JToolbarHelper::apply('user.apply');
            JToolbarHelper::save('user.save');

            if (!empty($this->item->user_id) && $canDo->get('user.edit')) {
                JToolBarHelper::custom('user.massEnableAccountAction', 'publish', '', 'Enable accounts');
                JToolBarHelper::custom('user.massDisableAccountAction', 'unpublish', '', 'Disable accounts');
                JToolbarHelper::custom('user.massDeleteDeviceAction', 'delete', '', 'COM_MOBILEASSISTANTCONNECTOR_N_ITEMS_DELETE');
            }
        }

        if (empty($this->item->user_id)) {
            JToolbarHelper::cancel('user.cancel');
        } else {
            JToolbarHelper::cancel('user.cancel', 'Close');
        }

        JToolbarHelper::divider();
    }

    public static function gridPublished($name, $i)
    {
        $published = JHtml::_('grid.published', $name, $i);

        return $published;
    }
}