<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

defined('_JEXEC') or die;
jimport('joomla.log.log');
jimport('joomla.application.component.view');

if (version_compare(JVERSION,'3.0', '>=')) {
    class MobileassistantconnectorViewUsersBase extends JViewLegacy {
        protected static function renderSidebar() {
            return JHtmlSidebar::render();
        }
    }
} else {
    class MobileassistantconnectorViewUsersBase extends JView {
        protected static function renderSidebar() {
            return '';
        }
    }
}

class MobileassistantconnectorViewUsers extends MobileassistantconnectorViewUsersBase {
    /**
     * The list of users
     *
     * @var    JObject
     * @since  1.6
     */
    protected $item;

    /**
     * The pagination object
     *
     * @var    JPagination
     * @since  1.6
     */
    protected $pagination;

    /**
     * The model state
     *
     * @var    JObject
     * @since  1.6
     */
    protected $state;

    /**
     * Execute and display a template script.
     *
     * @param   string  $tpl  The name of the template file to parse; automatically searches through the template paths.
     *
     * @return  mixed  A string if successful, otherwise an Error object.
     *
     * @since   1.6
     */

    public function display($tpl = null)
    {
        $this->items = $this->get('Items');
        $this->state = $this->get('State');

        MobileassistantconnectorHelper::addSubmenu('mobileassistantconnector');
        $this->sidebar = self::renderSidebar();

        // Check for errors.
        if (count($errors = $this->get('Errors')))
        {
            JError::raiseError(500, implode("\n", $errors));
            return false;
        }

        $this->addToolbar();
        parent::display($tpl);
    }

    protected function addToolbar()
    {
        $state = $this->get('State');
        $canDo = MobileassistantconnectorHelper::getActions($state->get('filter.category_id'), 'category', 'com_mobileassistantconnector');
        $user  = JFactory::getUser();

        // Get the toolbar object instance
        JToolbarHelper::title(JText::_('Mobile Assistant Connector'));

        if ($canDo->get('user.edit'))
        {
            JToolbarHelper::addNew('user.add');
        }

        if ($canDo->get('user.edit'))
        {
            JToolbarHelper::editList('user.edit');
        }

        if ($user->authorise('core.admin', 'com_mobileassistantconnector') || $user->authorise('core.options', 'com_mobileassistantconnector'))
        {
            JToolbarHelper::preferences('com_mobileassistantconnector');
        }

        if ($canDo->get('user.edit') && $canDo->get('user.view'))
        {
            JToolbarHelper::publish('users.publish', 'COM_MOBILEASSISTANTCONNECTOR_N_ITEMS_PUBLISHED', true);
            JToolbarHelper::unpublish('users.unpublish', 'COM_MOBILEASSISTANTCONNECTOR_N_ITEMS_UNPUBLISHED', true);
            JToolbarHelper::deleteList('JGLOBAL_CONFIRM_DELETE', 'users.delete', 'COM_MOBILEASSISTANTCONNECTOR_N_ITEMS_DELETE');
//            JToolbarHelper::deleteList('JGLOBAL_CONFIRM_DELETE', 'users.delete', 'COM_MOB_ASSISTANT_CONNECTOR_DELETE');
        }

        JToolbarHelper::divider();
    }
}

