<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

defined('_JEXEC') or die('Restricted access');

if (!class_exists( 'VmConfig' )) require(JPATH_ROOT . '/administrator/components/com_virtuemart/helpers/config.php');
if (!class_exists('CurrencyDisplay')) require (JPATH_ROOT . '/administrator/components/com_virtuemart/helpers/currencydisplay.php');
if(!class_exists('VirtueMartControllerInvoice')) require( JPATH_ROOT . '/components/com_virtuemart/controllers/invoice.php' );
VmConfig::loadConfig();

require_once( JPATH_SITE . '/components/com_mobileassistantconnector/helpers/access.php');
require_once( JPATH_SITE . '/components/com_mobileassistantconnector/helpers/device_and_push_notifications.php');
require_once JPATH_COMPONENT_ADMINISTRATOR.'/helpers/qrcode.php';

jimport('joomla.log.log');

class MobileassistantconnectorControllerMobileassistantconnector extends JControllerLegacy
{
    private $module_name = 'com_mobileassistantconnector';
    private $mob_api_key = '';
    private $call_function;
    private $shop_id;
    private $callback;
    private $hash = false;
    private $session_key;
    private $def_currency;
    private $def_lang;
    private $file_logger;
    private $cart_version;
    private $app_connection_id;
    private $registration_id;
    private $registration_id_old;
    private $api_key;
    private $push_new_order;
    private $push_order_statuses;
    private $push_new_customer;
    private $push_currency_code;
    private $account_email;
    private $device_name;
    private $device_unique_id;
    private $currency_code;
    private $action;
    private $product_id;
    private $statuses;
    private $orders_from;
    private $orders_to;
    private $search_order_id;
    private $page;
    private $show;
    private $get_statuses;
    private $order_id;
    private $only_items;
    private $user_id;
    private $new_status;
    private $carts_from;
    private $carts_to;
    private $cart_id;
    private $order_by;
    private $params;
    private $val;
    private $sort_by;
    private $group_by_product_id;
    private $search_carts;
    private $check_permission;

    const MODULE_VERSION   = '2';
    const CURRENCY_NOT_SET = 'not_set';
    const GUEST            = 'Guest not registered';

    public function registerDefaultTask($method)
    {
        parent::registerDefaultTask($method);

        JLog::addLogger(
            array(
                'text_file' => 'com_mobileassistantconnector.log.php',
                'text_file_path' => JFactory::getConfig()->get('log_path'),
                'text_entry_format' => '{DATETIME} {PRIORITY} {MESSAGE}'
            ),
            JLog::ALL,
            array('com_mobileassistantconnector')
        );

        VmConfig::loadJLang('com_virtuemart_orders', true);
        self::indexAction();
    }

    public function indexAction()
    {
        $config = JFactory::getConfig();

        if (!JComponentHelper::isEnabled($this->module_name)) {
            $this->generateOutput('module_disabled');
        }

        $xml = JFactory::getXML(JPATH_SITE .'/administrator/components/com_mobileassistantconnector/mobileassistantconnector.xml');
        $version = (string)$xml->version;
        $this->def_lang     = JComponentHelper::getParams('com_languages')->get('site');
        $this->def_currency = CurrencyDisplay::getInstance()->getId();
        $this->cart_version = $version;

        MobileassistantconnectorAccess::clearOldData();

        $this->callback = $this->getIsset('callback') ? JRequest::getString('callback') : null;
        $this->call_function = JRequest::get('call_function')['task'] ? JRequest::get('call_function')['task'] : '';
        $this->hash = $this->getIsset('hash') ? JRequest::getString('hash') : null;
        $this->session_key = $this->getIsset('key') ? JRequest::getString('key') : null;
        $this->device_unique_id = $this->getIsset('device_unique_id') ? JRequest::getString('device_unique_id') : null;
        $this->registration_id = $this->getIsset('registration_id') ? JRequest::getString('registration_id') : null;
        $this->account_email = $this->getIsset('account_email') ? JRequest::getString('account_email') : null;
        $this->device_name = $this->getIsset('device_name') ? JRequest::getString('device_name') : null;

        if ($this->getIsset('hash_only')) {
            $this->generateOutput('You should update Virtuemart Mobile Assistant application');
        }

        if (empty($this->call_function)) {
            $this->runSelfTest();
        }

        $this->updateDeviceLastActivity();

        if ($this->call_function == 'get_qr_code' && $this->hash) {
            $this->getQrCode($this->hash);
        }

        if ($this->call_function == 'get_version') {
            $this->getVersion();
        }

        if ($this->hash) {
            $key = MobileassistantconnectorAccess::getSessionKey($this->hash);

            if (!$key) {
                JLog::add(JText::_('Hash accepted is incorrect'), JLog::WARNING, 'com_mobileassistantconnector');
                $this->generateOutput('auth_error');
            }

            $this->generateOutput(array('session_key' => $key));
        } elseif ($this->session_key || $this->session_key === '') {
            if (!MobileassistantconnectorAccess::checkSessionKey($this->session_key)) {
                JLog::add(JText::_('Key accepted is incorrect'), JLog::WARNING, 'com_mobileassistantconnector');
                $this->generateOutput(array('bad_session_key' => true));
            }
        } else {
            MobileassistantconnectorAccess::addFailedAttempt();
            JLog::add(JText::_('Authorization error'), JLog::WARNING, 'com_mobileassistantconnector');
            $this->generateOutput('auth_error');
        }

        $request_params = $_REQUEST;
        $params = $this->validateTypes($request_params, array(
            'show'                => 'INT',
            'page'                => 'INT',
            'is_mail'             => 'INT',
            'search_order_id'     => 'STR',
            'orders_from'         => 'STR',
            'orders_to'           => 'STR',
            'customers_from'      => 'STR',
            'customers_to'        => 'STR',
            'date_from'           => 'STR',
            'date_to'             => 'STR',
            'graph_from'          => 'STR',
            'graph_to'            => 'STR',
            'stats_from'          => 'STR',
            'stats_to'            => 'STR',
            'products_to'         => 'STR',
            'products_from'       => 'STR',
            'order_id'            => 'INT',
            'user_id'             => 'INT',
            'params'              => 'STR',
            'val'                 => 'STR',
            'search_val'          => 'STR',
            'statuses'            => 'STR',
            'sort_by'             => 'STR',
            'last_order_id'       => 'STR',
            'product_id'          => 'INT',
            'get_statuses'        => 'INT',
            'cust_with_orders'    => 'INT',
            'data_for_widget'     => 'INT',
            'registration_id'     => 'STR',
            'registration_id_old' => 'STR',
            'api_key'             => 'STR',
            'push_new_order'      => 'INT',
            'push_order_statuses' => 'STR',
            'account_email'       => 'STR',
            'device_name'         => 'STR',
            'device_unique_id'    => 'STR',
            'push_new_customer'   => 'INT',
            'app_connection_id'   => 'INT',
            'push_currency_code'  => 'INT',
            'tracking_number'     => 'STR',
            'tracking_title'      => 'STR',
            'action'              => 'STR',
            'carrier_code'        => 'STR',
            'custom_period'       => 'INT',
            'shop_id'             => 'INT',
            'cart_id'             => 'INT',
            'new_status'          => 'STR',
            'currency_code'       => 'STR',
            'fc'                  => 'STR',
            'module'              => 'STR',
            'controller'          => 'STR',
            'search_carts'        => 'STR',
            'carts_from'          => 'STR',
            'carts_to'            => 'STR',
            'key'                 => 'STR',
            'only_items'          => 'INT',
            'order_by'            => 'STR',
            'group_by_product_id' => 'INT',
            'check_permission'    => 'STR',
        ));

        foreach ($params as $k => $value) {
            $this->{$k} = $value;

            if ($k == 'currency_code') {
                if ($value == self::CURRENCY_NOT_SET) {
                    $this->{$k} = $value;
                } elseif ($value == 'base_currency' || CurrencyDisplay::getInstance((int)$value) === null) {
                    $this->{$k} = (int)$config->get('PS_CURRENCY_DEFAULT');
                } else {
                    $this->{$k} = (int)$value;
                }
            }
        }

        $this->show = (empty($this->show) || $this->show < 1) ? 25 : $this->show;
        $this->page = (empty($this->page) || $this->page < 1) ? 1 : $this->page;

        if ($this->call_function == 'test_config') {
            $result = array('test' => 1);

            if ($this->check_permission) {
                $this->call_function = $this->check_permission;
                $result['permission_granted'] = $this->isActionAllowed() ? 1 : 0;
            }

            $this->generateOutput($result);
        }

        $this->checkAllowedActions();
        $result = $this->getMethodResult($this->call_function);

        if ($this->call_function == 'get_order_pdf') {
            die($result);
        } else {
            $this->generateOutput($result);
        }
    }

    private function getMethodResult($call_function)
    {
        $result = false;

        switch ($call_function) {
            case 'run_self_test':
                $this->runSelfTest();
                break;
            case 'get_currencies':
                $result = $this->getCurrencies();
                break;
            case 'get_store_title':
                $result = $this->getStoreTitle();
                break;
            case 'push_notification_settings':
                $result = $this->pushNotificationSettings();
                break;
            case 'get_orders_statuses':
                $result = $this->getOrdersStatuses();
                break;
            case 'get_store_stats':
                $result = $this->getStoreStats();
                break;
            case 'get_data_graphs':
                $result = $this->getDataGraphs();
                break;
            case 'get_status_stats':
                $result = $this->getStatusStats();
                break;
            case 'search_products':
                $result = $this->searchProducts();
                break;
            case 'search_products_ordered':
                $result = $this->searchProductsOrdered();
                break;
            case 'get_products_info':
                $result = $this->getProductsInfo();
                break;
            case 'get_products_descr':
                $result = $this->getProductsDescr();
                break;
            case 'get_customers':
                $result = $this->getCustomers();
                break;
            case 'get_customers_info':
                $result = $this->getCustomersInfo();
                break;
            case 'get_orders':
                $result = $this->getOrders();
                break;
            case 'get_orders_info':
                $result = $this->getOrdersInfo();
                break;
            case 'set_order_action':
                $result = $this->setOrderAction();
                break;
            case 'get_carriers':
                $result = $this->getCarriers();
                break;
            case 'get_abandoned_carts_list':
                $result = $this->getAbandonedCarts();
                break;
            case 'get_abandoned_cart_details':
                $result = $this->getAbandonedCartInfo();
                break;
            case 'get_order_pdf':
                $result = $this->getOrderPdf();
                break;
            case 'delete_push_config':
                $result = $this->deletePushConfig();
                break;
            default:
                JLog::add(JText::_('Unknown method call ({$this->call_function})'), JLog::WARNING, 'com_mobileassistantconnector');
                $this->generateOutput('old_module');
                break;
        }

        return $result;
    }

    private function getCurrencies()
    {
        $currency_ret = array();
        $currency_ids = array();
        $currency_inf = array();
        $currency_model = VmModel::getModel('currency');
        $currencies = $currency_model->getVendorAcceptedCurrrenciesList();
        $i = 0;

        foreach ($currencies as $currency) {
            if (!in_array($currency->virtuemart_currency_id, $currency_ids)) {
                $currency_ret[$i]['code'] = $currency->virtuemart_currency_id;
                $currency_inf[$i] = $currency_model->getCurrency($currency->virtuemart_currency_id);
                foreach ($currency_inf as $value) {
                    $currency_ret[$i]['symbol'] = $value->currency_symbol;
                    $currency_ret[$i]['name'] = $value->currency_name;
                }
                $currency_ids[] = $currency->virtuemart_currency_id;
                $i++;
            }
        }

        return $currency_ret;
    }

    private function getStoreTitle()
    {
        $config = new JConfig();
        $this->shop_info = $config->sitename;
        return array('test' => 1, 'title' => $this->shop_info);
    }

    private function pushNotificationSettings()
    {
        $db = JFactory::getDbo();
        if ((int)$this->app_connection_id < 1) {
            return false;
        }

        if (!$this->registration_id && !$this->api_key && !$this->device_unique_id) {
            return array('error' => 'missing_parameters');
        }

        // Update old registration id
        if ($this->registration_id_old) {
            $query = $db->getQuery(true);
            $query
                ->update($db->quoteName('#__mobassistantconnector_push_notifications'))
                ->set($db->quoteName('device_id') . ' = ' . $this->registration_id)
                ->where($db->quoteName('device_id') . ' = ' . JFilterInput::getInstance()->clean($this->registration_id_old));
            $db->setQuery($query);
            $db->execute();
        }

        $result        = array('success' => 'true');
        $device_name   = $this->device_name ? $this->device_name : '';
        $date          = date('Y-m-d H:i:s');
        $account_id    = $this->getAccountIdByEmail((string)$this->account_email);
        $query2 = $query3 = $query4 = $db->getQuery(true);

        $currency_code  = $this->def_currency;
        if ($this->push_currency_code) {
            $currency_code = $this->push_currency_code;
        } elseif ($this->currency_code) {
            $currency_code = (int)$this->currency_code;
        }

        $query1="INSERT INTO #__mobassistantconnector_devices (`device_unique`, `account_id`,
            `device_name`, `last_activity`) VALUES ('".JFilterInput::getInstance()->clean($this->device_unique_id).
            "',".(int)$account_id.",'" .JFilterInput::getInstance()->clean($device_name)."','".
            JFilterInput::getInstance()->clean($date)."') ON DUPLICATE KEY UPDATE `device_name` = '" .
            JFilterInput::getInstance()->clean($device_name) . "', `last_activity` = '".
            JFilterInput::getInstance()->clean($date)."'";
        $db->setQuery($query1);
        $db->execute();

        $id = $db->getQuery(true);
        $id
            ->select($db->quoteName('device_unique_id'))
            ->from($db->quoteName('#__mobassistantconnector_devices'))
            ->where('device_unique = "'. JFilterInput::getInstance()->clean($this->device_unique_id)
                . '" AND account_id = ' .(int)$account_id);

        $db->setQuery($id);
        $id = $db->loadResult();
        $user_id = (int)MobileassistantconnectorAccess::getUserIdBySessionKey($this->session_key);
        $user_actions = MobileassistantconnectorAccess::getAllowedActionsByUserId($user_id);

        // Delete empty record
        if ($this->push_new_order == 0 && empty($this->push_order_statuses) && $this->push_new_customer == 0) {
            $query2
                ->delete($db->quoteName('#__mobassistantconnector_push_notifications'))
                ->where($db->quoteName('device_id') . ' = ' . $this->registration_id . ' AND ' .
                    $db->quoteName('app_connection_id') . ' = ' . (int)$this->app_connection_id);
            $db->setQuery($query2);

            MobileassistantconnectorHelperDevice_and_push_notifications::deleteEmptyDevices();
            MobileassistantconnectorHelperDevice_and_push_notifications::deleteEmptyAccounts();

        } elseif (
            !empty($user_actions) && (in_array('push_notification_settings_new_order', $user_actions)
                || in_array('push_notification_settings_new_customer', $user_actions)
                || in_array('push_notification_settings_order_statuses', $user_actions))
        ) {
            $id_exists = $db->getQuery(true);
            $id_exists
                ->select($db->quoteName('id'))
                ->from($db->quoteName('#__mobassistantconnector_push_notifications'))
                ->where('device_id = "' . JFilterInput::getInstance()->clean($this->registration_id) .
                    '" AND app_connection_id = ' .(int)$this->app_connection_id );
            $db->setQuery($id_exists);
            $id_exists = $db->loadResult();

            $data = array(
                'id'               => 'null',
                'device_unique_id' => (int)$id,
                'user_id'          => $user_id,
                'device_id'        => '"' . $this->registration_id . '"',
            );

            $data_exist = array(
                $db->quoteName('device_unique_id') . ' = ' .  $db->quote((int)$id),
                $db->quoteName('user_id') . ' = ' . $db->quote($user_id),
                $db->quoteName('device_id') . ' = ' . $db->quote($this->registration_id)
            );

            if (in_array('push_notification_settings_new_order', $user_actions)) {
                array_push($data_exist, $db->quoteName('new_order') . ' = ' . $db->quote((int)$this->push_new_order));
                $data['new_order'] = (int)$this->push_new_order;
            }

            /*if (in_array('push_notification_settings_order_statuses', $user_actions)) {
                $data['order_statuses'] = (string)$this->push_order_statuses;
            } */

            if (in_array('push_notification_settings_new_customer', $user_actions)) {
                array_push($data_exist, $db->quoteName('new_customer') . ' = ' . $db->quote((int)$this->push_new_customer));
                $data['new_customer'] = (int)$this->push_new_customer;
            }

            if ($id_exists) {
                $query3
                    ->update($db->quoteName('#__mobassistantconnector_push_notifications'))
                    ->set($data_exist)
                    ->where($db->quoteName('id') . ' = ' . (int)$id_exists);
                $db->setQuery($query3);
                $db->execute();

            } else {
                $data['order_statuses'] = 'null';
                $data['app_connection_id'] = (int)$this->app_connection_id;
                $data['id_shop'] = -1;
                $data['currency_code'] = (int)$currency_code;
                $data = implode(",", $data);
                $query3
                    ->insert($db->quoteName('#__mobassistantconnector_push_notifications'))
                    ->values($data);
                $db->setQuery($query3);
                $db->execute();
            }
        }

        if (!$query1 || !$query2 || !$query3 || !$query4) {
            $result = array('error' => 'could_not_update_data');
        }

        return $result;
    }


    private function checkAllowedActions()
    {
        if (!$this->isActionAllowed()) {
            $this->generateOutput('action_forbidden');
        }
    }

    private function isActionAllowed()
    {
        $allowed_functions_always = $this->getAlwaysAllowedFunctions();

        if (in_array($this->call_function, $allowed_functions_always)) {
            return true;
        }

        $allowed_actions = MobileassistantconnectorAccess::getAllowedActionsBySessionKey($this->session_key);
        $restricted_actions_to_functions = MobileassistantconnectorAccess::getRestrictedActionsToFunctions();
        $is_allowed = false;

        if ($this->call_function == 'set_order_action') {
            if ($this->action == 'change_status' && in_array('update_order_status', $allowed_actions)) {
                $is_allowed = true;
            } elseif ($this->action == 'update_track_number'
                && in_array('update_order_tracking_number', $allowed_actions)) {
                $is_allowed = true;
            }
        } else {
            foreach ($restricted_actions_to_functions as $key => $values) {
                if (in_array($this->call_function, $values)) {
                    if (in_array($key, $allowed_actions)) {
                        $is_allowed = true;
                    }

                    break;
                }
            }
        }

        return $is_allowed;
    }

    private function updateDeviceLastActivity()
    {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);

        if (!$this->device_unique_id) {
            return;
        }

        $account_id = $this->getAccountIdByEmail((string)$this->account_email);
        $query
            ->update($db->quoteName('#__mobassistantconnector_devices'))
            ->set("last_activity = '" . date('Y-m-d H:i:s') . "'")
            ->where("device_unique = '" . JFilterInput::getInstance()->clean($this->device_unique_id).
                "' AND `account_id` = ".(int)$account_id);
        $db->setQuery($query);
        $db->execute();
    }

    private function getOrdersStatuses()
    {
        $statuses = VmModel::getModel('orderstatus')->getOrderStatusList();
        $statuses_ret = array();
        $i=0;

        foreach ($statuses as $status) {
            $statuses_ret[$i]['st_id'] = $status->order_status_code;
            $statuses_ret[$i]['st_name'] = VmText::_($status->order_status_name);
            $i++;
        }

        return $statuses_ret;
    }

    private function getStoreStats()
    {
        $data_graphs = array();
        $query_where_parts = array();
        $order_status_stats = array();
        $store_stats = array(
            'count_orders'    => '0',
            'total_sales'     => '0',
            'count_customers' => '0',
            'count_products'  => '0',
            'last_order_id'   => '0',
            'new_orders'      => '0'
        );
        $today = date('Y-m-d', time());
        $date_from = $date_to = $today;

        if (!empty($this->stats_from)) {
            $date_from = $this->stats_from;
        }

        if (!empty($this->stats_to)) {
            $date_to = $this->stats_to;
        }

        if (isset($this->custom_period) && JString::strlen($this->custom_period) > 0) {
            $custom_period = $this->getCustomPeriod($this->custom_period);

            $date_from = $custom_period['start_date'];
            $date_to = $custom_period['end_date'];
        }

        if (!empty($date_from)) {
            $query_where_parts[] = "o.created_on >= '".date('Y-m-d H:i:s', strtotime($date_from.' 00:00:00'))."'";
        }

        if (!empty($date_to)) {
            $query_where_parts[] = "o.created_on <= '".date('Y-m-d H:i:s', strtotime($date_to.' 23:59:59'))."'";
        }

        if (!empty($this->statuses)) {
            $statuses = str_replace('|', "','", $this->statuses);
            $query_where_parts[] = " o.order_status IN ('"
                . JFilterInput::getInstance()->clean($statuses)."')";
        }

        // Get count of orders
        $db = JFactory::getDbo();
        $orders_count_obj = $db->getQuery(true);

        $orders_count_obj
            ->select('o.virtuemart_order_id AS count_orders, 
                o.order_total AS total,
                o.user_currency_id')
            ->from($db->quoteName('#__virtuemart_orders', 'o'));

        if (!empty($query_where_parts)) {
            $orders_count_obj->where(implode('AND ',$query_where_parts));
        }

        $db->setQuery($orders_count_obj);
        $orders_count = $db->loadAssocList();

        foreach($orders_count as $order) {
            // Convert price for each order
            $order['total'] = $this->convertPrice(
                $order['total'],
                $order['user_currency_id'],
                $this->currency_code
            );
            $orders_count['total_sales'] += $order['total'];
        }

        $count_of_orders = count($orders_count);
        if ($count_of_orders == 0) {
            $orders_count['total_sales'] = 0;
        }
        $orders_count['count_orders'] = $count_of_orders;

        // Form price
        $orders_count['total_sales'] = $this->formatPrice($orders_count['total_sales'], $this->currency_code);

        // Add count and sum of orders into return array
        $store_stats['count_orders'] = !empty($orders_count['count_orders']) ? $orders_count['count_orders'] : 0;
        $store_stats['total_sales'] = $orders_count['total_sales'];

        // Get count of products
        $products_count_obj = $db->getQuery(true);
        $products_count_obj
            ->select('COUNT(od.virtuemart_product_id) AS count_products')
            ->from($db->quoteName('#__virtuemart_orders', 'o'))
            ->join('LEFT',$db->quoteName('#__virtuemart_order_items', 'od').' ON( ' .
                $db->quoteName('od.virtuemart_order_id') . ' = ' . $db->quoteName('o.virtuemart_order_id') .')');

        if (!empty($query_where_parts)) {
            $products_count_obj->where(implode(' AND ', $query_where_parts));
        }
        $db->setQuery($products_count_obj);
        $products_count = $db->loadAssoc();

        // Add count and sum of products into return array
        $store_stats['count_products'] = !empty($products_count['count_products']) ? $products_count['count_products'] : 0;

        // Get last_order_id and count of new orders
        if ($this->last_order_id != '') {
            $order_last_obj = $db->getQuery(true);
            $order_last_obj
                ->select('COUNT(o.virtuemart_order_id) AS count_orders, MAX(o.virtuemart_order_id) AS last_order_id')
                ->from($db->quoteName('#__virtuemart_orders', 'o'));

            if (!empty($query_where_parts)) {
                $order_last_obj->where(
                    implode(' AND ', $query_where_parts).' AND o.virtuemart_order_id > '.(int)$this->last_order_id
                );
            } else {
                $order_last_obj->where('o.virtuemart_order_id > '.(int)$this->last_order_id);
            }
            $db->setQuery($order_last_obj);
            $order_last = $db->loadAssocList();
            $order_last = array_shift($order_last);

            // Add last_order_id and count of new orders into return array
            if ($order_last['last_order_id'] > (int)$this->last_order_id) {
                $store_stats['last_order_id'] = $order_last['last_order_id'];
            }
            $store_stats['new_orders'] = $order_last['count_orders'];
        }

        unset($query_where_parts);

        if (!empty($date_from)) {
            $query_where_parts[] = "u.registerDate >= '".date('Y-m-d H:i:s', (strtotime($date_from.' 00:00:00')))."'";
        }

        if (!empty($date_to)) {
            $query_where_parts[] = "u.registerDate <= '".date('Y-m-d H:i:s', (strtotime($date_to.' 23:59:59')))."'";
        }

        // Get count of customers
        $customers_count_obj = $db->getQuery(true);
        $customers_count_obj
            ->select('COUNT(u.id) AS count_customers')
            ->from($db->quoteName('#__users', 'u'))
            ->leftJoin('#__virtuemart_vmusers AS vm ON u.id = vm.virtuemart_user_id')
            ->join('LEFT OUTER','(SELECT COUNT(virtuemart_order_id) AS total_orders, virtuemart_user_id 
              FROM #__virtuemart_orders GROUP BY virtuemart_user_id) AS tot ON tot.virtuemart_user_id = u.id');

        if (!empty($query_where_parts)) {
            $customers_count_obj->where(implode(' AND ', $query_where_parts));
        }

        $db->setQuery($customers_count_obj);
        $customers_count = $db->loadResult();

        // Add count of customers into return array
        $store_stats['count_customers'] = !empty($customers_count) ? $customers_count : 0;

        if (!isset($this->data_for_widget) || empty($this->data_for_widget) || $this->data_for_widget != 1) {
            $data_graphs = $this->getDataGraphs();
        }

        if (!isset($this->data_for_widget) || $this->data_for_widget != 1) {
            $order_status_stats = $this->getStatusStats();
        }

        $result = array_merge(
            $store_stats,
            array('data_graphs' => $data_graphs),
            array('order_status_stats' => $order_status_stats)
        );

        return $result;
    }

    private function getDataGraphs()
    {
        $db = JFactory::getDbo();
        $orders = array();
        $customers = array();
        $where_statuses = '';
        $currency = CurrencyDisplay::getInstance($this->currency_code);
        $average = array(
            'avg_sum_orders' => 0,
            'sum_orders'     => 0,
            'tot_orders'     => 0,
            'tot_customers'  => 0,
            'avg_orders'     => 0,
            'avg_customers'  => 0,
            'avg_cust_order' => '0.00',
        );

        if (empty($this->graph_from)) {
            $this->graph_from = date('Y-m-d', mktime(0, 0, 0, date('m'), date('d') - 7, date('Y')));
        }

        if (empty($this->graph_to)) {
            if (!empty($this->stats_to)) {
                $this->graph_to = $this->stats_to;
            } else {
                $this->graph_to = date('Y-m-d', time());
            }
        }

        $start_date = $this->graph_from.' 00:00:00';
        $end_date = $this->graph_to.' 23:59:59';
        $plus_date = '+1 day';
        $group_by_period = 'day';

        if (isset($this->custom_period) && JString::strlen($this->custom_period) > 0) {
            $custom_period = $this->getCustomPeriod($this->custom_period);

            if ($this->custom_period == 3) {
                $plus_date = '+3 day';
            } elseif ($this->custom_period == 4 || $this->custom_period == 8) {
                $plus_date = '+1 week';
                $group_by_period = 'week';
            } elseif ($this->custom_period == 5 || $this->custom_period == 6 || $this->custom_period == 7) {
                $plus_date = '+1 month';
                $group_by_period = 'month';
            }

            if ($this->custom_period == 7) {
                $dates_obj = $db->getQuery(true);
                $dates_obj
                    ->select('MIN(created_on) AS min_date_add, MAX(created_on) AS max_date_add')
                    ->from('#__virtuemart_orders');
                $db->setQuery($dates_obj);
                $dates = $db->loadAssocList();

                $dates = array_shift($dates);
                $start_date = $dates['min_date_add'];
                $end_date = $dates['max_date_add'];
            } else {
                $start_date = $custom_period['start_date'].' 00:00:00';
                $end_date = $custom_period['end_date'].' 23:59:59';
            }
        }

        $start_date = strtotime($start_date);
        $end_date = strtotime($end_date);
        $date = $start_date;
        $d = 0;

        if (!empty($this->statuses)) {
            $statuses = str_replace('|', "','", $this->statuses);
            $where_statuses = "o.order_status IN ('"
                .JFilterInput::getInstance()->clean($statuses)."') AND ";
        }

        while ($date <= $end_date) {
            $d++;
            $date_str = date('Y-m-d H:i:s', ($date));
            $where_orders_date_add = "o.created_on >= '{$date_str}' AND o.created_on < '"
                .date('Y-m-d H:i:s', (strtotime($plus_date, $date)))."'";

            // Get orders
            $orders_tot_obj = $db->getQuery(true);
            $orders_tot_obj
                ->select('
                o.created_on,
                SUM(o.order_total) AS value,
                COUNT(o.virtuemart_order_id) AS tot_orders,
                o.user_currency_id
                ')
                ->from($db->quoteName('#__virtuemart_orders', 'o'))
                ->where('('. $where_statuses.$where_orders_date_add .')')
                ->group('DATE(o.created_on)')
                ->order($db->quoteName('o.created_on'));
            $db->setQuery($orders_tot_obj);
            $orders_tot = $db->loadAssocList();

            // Get total of orders per date
            $total_order_per_day = 0;

            foreach ($orders_tot as $order) {
                $total_order_per_day += $order['value'];
                $average['tot_orders'] += $order['tot_orders'];
                $average['sum_orders'] += $order['value'];
            }

            // Convert price
            $total_order_per_day = $this->convertPrice(
                $total_order_per_day,
                $orders_tot['user_currency_id'],
                $this->currency_code
            );

           /* $total_order_per_day = $currency->convertCurrencyTo(
                $this->currency_code,
                $total_order_per_day,
                $this->currency_code == $this->def_currency
            );*/

            $orders[] = array($date * 1000, $total_order_per_day);
            $where_customers_date_add = "created_on >= '{$date_str}' AND created_on < '"
                .date('Y-m-d H:i:s', strtotime($plus_date, $date))."'";

            // Get customers
            $customers_tot_obj = $db->getQuery(true);
            $customers_tot_obj
                ->select('
                COUNT(virtuemart_user_id) AS tot_customers,
                created_on
            ')
                ->from($db->quoteName('#__virtuemart_vmusers'))
                ->where($where_customers_date_add)
//                ->where($where_customers_date_add.$where_id_shop)
                ->group('DATE(created_on)')
                ->order($db->quoteName('created_on'));
            $db->setQuery($customers_tot_obj);
            $customers_tot = $db->loadAssocList();

            $total_customer_per_day = 0;

            // Get total of customers per date
            foreach ($customers_tot as $customer) {
                $total_customer_per_day += $customer['tot_customers'];
                $average['tot_customers'] += $customer['tot_customers'];
            }

            $customers[] = array($date * 1000, $total_customer_per_day);
            $date = strtotime($plus_date, $date);
        }

        // Add 2 additional element into array of orders for graph in mobile application
        if (count($orders) == 1) {
            $orders_tmp = $orders[0];
            $orders = array();
            $orders[0][] = strtotime(date('Y-m-d', $orders_tmp[0] / 1000).'-1 month') * 1000;
            $orders[0][] = 0;
            $orders[1] = $orders_tmp;
            $orders[2][] = strtotime(date('Y-m-d', $orders_tmp[0] / 1000).'+1 month') * 1000;
            $orders[2][] = 0;
        }

        // Add 2 additional element into array of customers for graph in mobile application
        if (count($customers) == 1) {
            $customers_tmp = $customers[0];
            $customers = array();
            $customers[0][] = strtotime(date('Y-m-d', $customers_tmp[0] / 1000).'-1 month') * 1000;
            $customers[0][] = 0;
            $customers[1] = $customers_tmp;
            $customers[2][] = strtotime(date('Y-m-d', $customers_tmp[0] / 1000).'+1 month') * 1000;
            $customers[2][] = 0;
        }

        if ($d <= 0) {
            $d = 1;
        }

        $average['avg_sum_orders'] = $this->formatPrice($average['sum_orders'] / $d, $this->currency_code);
        $average['avg_orders']     = number_format($average['tot_orders'] / $d, 1, '.', ' ');
        $average['avg_customers']  = number_format($average['tot_customers'] / $d, 1, '.', ' ');

        if ($average['tot_customers'] > 0) {
            $average['avg_cust_order'] = $average['sum_orders'] / $average['tot_customers'];
        }

        $average['avg_cust_order'] = $this->formatPrice($average['avg_cust_order'], $this->currency_code);
        $average['sum_orders']    = $this->formatPrice($average['sum_orders'], $this->currency_code);
        $average['tot_customers'] = number_format($average['tot_customers'], 1, '.', ' ');
        $average['tot_orders']    = number_format($average['tot_orders'], 1, '.', ' ');
		$average['group_by']      = $group_by_period;

        if ($this->currency_code != self::CURRENCY_NOT_SET && $this->currency_code > 0) {
            $currency = VmModel::getModel('currency')->getCurrency($this->currency_code);
        } else {
            $currency = VmModel::getModel('currency')->getCurrency($this->def_currency);
        }

        return array(
            'orders' => $orders,
            'customers' => $customers,
            'currency_sign' => $currency->currency_symbol,
            'average' => $average
        );
    }

    private function getStatusStats()
    {
        $db = JFactory::getDbo();
        $order_statuses_ret = array();
        $query_where_parts = array();
        $today = date('Y-m-d', time());
        $date_from = $date_to = $today;

        if (!empty($this->stats_from)) {
            $date_from = $this->stats_from;
        }

        if (!empty($this->stats_to)) {
            $date_to = $this->stats_to;
        }

        if (isset($this->custom_period) && JString::strlen($this->custom_period) > 0) {
            $custom_period = $this->getCustomPeriod($this->custom_period);

            $date_from = $custom_period['start_date'];
            $date_to = $custom_period['end_date'];
        }

        if (!empty($date_from)) {
            $date_from = $date_from.' 00:00:00';
        }

        if (!empty($date_to)) {
            $date_to = $date_to.' 23:59:59';
        }

        if (!empty($date_from) && !empty($date_to)) {
            $query_where_parts[] = "o.created_on >= '".date('Y-m-d H:i:s', strtotime($date_from))
                ."' AND o.created_on <= '".date('Y-m-d H:i:s', strtotime($date_to))."'";
        }

        // Get count of orders by status
        $order_statuses_obj = $db->getQuery(true);
        $order_statuses_obj
            ->select('
            o.virtuemart_order_id,
            o.order_total AS total,
            os.order_status_name AS name,
            o.order_status AS code,
            o.user_currency_id
        ')
            ->from($db->quoteName('#__virtuemart_orders', 'o'))
            ->join('LEFT', '#__virtuemart_orderstates AS os  ON (' . $db->quoteName('os.order_status_code')
                . ' = ' . $db->quoteName('o.order_status') .')');

        if (!empty($query_where_parts)) {
            $order_statuses_obj->where(implode(' AND ', $query_where_parts));
        }

        $db->setQuery($order_statuses_obj);
        $order_statuses = $db->loadAssocList();

        foreach ($order_statuses as $order_status) {
            $order_status['total'] = $this->convertPrice($order_status['total'], $order_status['user_currency_id'], $this->currency_code);
            $order_status['name'] = vmText::_($order_status['name']);
            $order_statuses_ret[$order_status['name']]['total'] += $order_status['total'];
            $order_statuses_ret[$order_status['name']]['count'] += 1;
            $order_statuses_ret[$order_status['name']]['code'] = $order_status['code'];
            $order_statuses_ret[$order_status['name']]['name'] = $order_status['name'];
        }

        $result = array();
        $k = 0;
        foreach ($order_statuses_ret as $order){
            $result[$k]['total'] = $this->formatPrice($order['total'], $this->currency_code);
            $result[$k]['name'] = $order['name'];
            $result[$k]['count'] = $order['count'];
            $result[$k]['code'] = $order['code'];
            $k++;
        }

        return $result;
    }

    private function searchProducts()
    {
        if (JVERSION < '3.5.1') {
            $def_lang_name = VmConfig::$vmlang;
        } else {
            $def_lang_name = VmConfig::$jDefLang;
        }
        $db = JFactory::getDbo();
        $query_where_parts = array();
        $products = array();
        $order_by = '';

        if (empty($this->sort_by)) {
            $this->sort_by = 'id';
        }

        switch ($this->sort_by) {
            case 'id':
                $order_by = 'p.virtuemart_product_id '.$this->getSortDirection('DESC');
                break;
            case 'name':
                $order_by = 'pl.product_name '.$this->getSortDirection('ASC');
                break;
            case 'qty':
                $order_by = 'p.product_in_stock '.$this->getSortDirection('DESC');
                break;
            case 'price':
                $this->sort_by = 'price';
                break;
            case 'status':
                $order_by = 'p.published '.$this->getSortDirection('DESC');
                break;
        }

        if (!empty($this->params) && !empty($this->val)) {
            $this->params = explode('|', $this->params);
            $count_params = count($this->params);

            for ($i = 0; $i < $count_params; $i++) {
                switch ($this->params[$i]) {
                    case 'pr_id':
                        if (!empty($this->val) && preg_match('/^\d+(?:,\d+)*$/', $this->val)) {
                            $query_where_parts[] = 'p.virtuemart_product_id IN ('.JFilterInput::getInstance()->clean($this->val).')';
                        } else {
                            $query_where_parts[] = 'p.virtuemart_product_id = '.(int)$this->val;
                        }
                        break;
                    case 'pr_sku':
                        $query_where_parts[] = "p.product_sku = '".JFilterInput::getInstance()->clean($this->val)."'";
                        break;
                    case 'pr_name':
                        $query_where_parts[] = "pl.product_name LIKE '%".JFilterInput::getInstance()->clean($this->val)."%'";
                        break;
                    case 'pr_desc':
                        $query_where_parts[] = "pl.product_desc LIKE '%".JFilterInput::getInstance()->clean($this->val)."%'";
                        break;
                    case 'pr_short_desc':
                        $query_where_parts[] = "pl.product_s_desc LIKE '%".JFilterInput::getInstance()->clean($this->val)."%'";
                        break;
                    default:

                        $values = explode('|', $this->val);
                        $count  = count($values);

                        for ($k = 0; $k < $count; $k++) {
                            if (empty($values[$k])) {
                                unset($values[$k]);
                            }
                        }

                        $values = "'".implode("','", $values)."'";
                        $params = explode('|', $this->params[$i]);
                        $count = count($params);

                        for ($k = 0; $k < $count; $k++) {
                            if (empty($params[$k])) {
                                continue;
                            }
                            $query_where_parts[] = "p.{$params[$k]} IN ($values)";
                        }
                }
            }
        }

        // Get products
        $products_obj = $db->getQuery(true);
        $products_obj->select('
            p.virtuemart_product_id AS product_id,
            pl.product_name AS name,
            p.product_sku AS sku,
            p.product_gtin,
            p.product_in_stock AS quantity,
            p.published AS status_code
        ')
            ->from($db->quoteName('#__virtuemart_products', 'p'))
            ->leftJoin(
                '#__virtuemart_products_' . $def_lang_name
                . ' AS pl ON (p.virtuemart_product_id = pl.virtuemart_product_id)'
            );

        if (!empty($query_where_parts)) {
            $products_obj->where(implode(' OR ', $query_where_parts));
        }

        if (!empty($order_by)) {
            $products_obj->order($order_by);
        }

        if ($this->sort_by != 'price') {
            $db->setQuery($products_obj, ($this->page - 1) * $this->show, $this->show);
        } else {
            $db->setQuery($products_obj);
        }

        $products_res = $db->loadAssocList();
        $product_model = VmModel::getModel('product');
        $currency = CurrencyDisplay::getInstance($this->currency_code);

        for ($i = 0; $i < count($products_res); $i++) {
            $product_info = $product_model->getProduct((int) $products_res[$i]['product_id'], true, true, false);

            if ($product_info) {
                $products_res[$i]['price'] = $this->convertPrice(
                    $product_info->prices['basePrice'],
                    $product_info->prices['product_currency'],
                    $this->currency_code
                );
                /*$products_res[$i]['price'] = $currency->convertCurrencyTo(
                    $this->currency_code,
                    $product_info->prices['basePrice'],
                    $this->currency_code == $this->def_currency
                );*/

                $product_image_id = $product_info->virtuemart_media_id[0];
                $image_url = JURI::root() . VmModel::getModel('media')->getData($product_image_id)->file_url;

                if ($image_url && $image_url != JURI::root()) {
                    $products_res[$i]['product_image'] = $image_url;
                }

                $products[] = $products_res[$i];
            }
        }

        if ($this->sort_by == 'price') {
            usort($products, array($this, 'sortItems'));
            $product_from = ($this->page - 1) * $this->show;
            $product_to = $this->show + 1;
            $products = array_slice($products, $product_from, $product_to);
        }

        for ($i = 0; $i < count($products); $i++) {
            $products[$i]['price'] = $this->formatPrice(
                $products[$i]['price'],
                $this->currency_code
            );
        }

        // Get count of products
        $products_count_obj = $db->getQuery(true);
        $products_count_obj
            ->select('COUNT(p.virtuemart_product_id) AS count_prods')
            ->from($db->quoteName('#__virtuemart_products', 'p'))
            ->leftJoin('#__virtuemart_products_' . $def_lang_name
                . ' AS pl ON (p.virtuemart_product_id = pl.virtuemart_product_id )'
            );

        if (!empty($query_where_parts)) {
            $products_count_obj->where(implode(' OR ', $query_where_parts));
        }

        $db->setQuery($products_count_obj);
        $products_count = $db->loadAssocList();
        $products_count = array_shift($products_count);

        return array('products_count' => (int)$products_count['count_prods'], 'products' => $products);
    }

    private function searchProductsOrdered()
    {
		if (JVERSION < '3.5.1') {
            $def_lang_name = VmConfig::$vmlang;
        } else {
            $def_lang_name = VmConfig::$jDefLang;
        }
        $db = JFactory::getDbo();
        $query_where_parts = array();
        $query_where_parts_search = array();
        $products = array();
        $order_by = '';
        $product_model = VmModel::getModel('product');
//        $this->shop_id = (int)$this->shop_id;

        if (!empty($this->params) && !empty($this->val)) {
            $this->params = explode('|', $this->params);
            $count_params = count($this->params);

            for ($i = 0; $i < $count_params; $i++) {
                switch ($this->params[$i]) {
                    case 'pr_id':
                        if (!empty($this->val) && preg_match('/^\d+(?:,\d+)*$/', $this->val)) {
                            $query_where_parts_search[] = 'oi.virtuemart_product_id IN ('
                                . JFilterInput::getInstance()->clean($this->val) . ')';
                        } else {
                            $query_where_parts_search[] = 'oi.virtuemart_product_id = ' . (int)$this->val;
                        }
                        break;
                    case 'pr_sku':
                        $query_where_parts_search[] = "oi.order_item_sku = '"
                            . JFilterInput::getInstance()->clean($this->val) . "'";
                        break;
                    case 'pr_name':
                        $query_where_parts_search[] = "oi.order_item_name LIKE '%"
                            . JFilterInput::getInstance()->clean($this->val) . "%'";
                        break;
                    case 'order_id':
                        $query_where_parts_search[] = 'oi.virtuemart_order_id = ' . (int)$this->val;
                        break;
                }
            }
        }

        if (!empty($query_where_parts_search)) {
            $query_where_parts[] = implode(' OR ', $query_where_parts_search);
        }

        if (!empty($this->products_from)) {
            $query_where_parts[] = "o.created_on >= '" . date('Y-m-d H:i:s', strtotime($this->products_from . ' 00:00:00'))
                . "'";
        }

        if (!empty($this->products_to)) {
            $query_where_parts[] = "o.created_on <= '" . date('Y-m-d H:i:s', strtotime($this->products_to . ' 23:59:59'))
                . "'";
        }

        if (!empty($this->statuses)) {
            $statuses = str_replace('|', "','", $this->statuses);
            $query_where_parts[] = "o.order_status IN ('" . JFilterInput::getInstance()->clean($statuses) . "')";
        }

        if (empty($this->sort_by)) {
            $this->sort_by = 'id';
        }

        switch ($this->sort_by) {
            case 'id':
                $order_by = 'oi.virtuemart_product_id ' . $this->getSortDirection('DESC');
                break;
            case 'name':
                $order_by = 'pl.product_name ' . $this->getSortDirection('ASC');
                break;
            case 'qty':
                $order_by = 'quantity ' . $this->getSortDirection('DESC');
                break;
            case 'total':
                $this->sort_by = 'price';
                break;
        }

        // Get ordered products
        $products_ordered_obj = $db->getQuery(true);
        $products_ordered_obj
            ->from($db->quoteName('#__virtuemart_order_items', 'oi'))
            ->leftJoin(
                '#__virtuemart_products_' . $def_lang_name
                . ' AS pl ON (oi.virtuemart_product_id = pl.virtuemart_product_id)'
            )
            ->leftJoin(
                '#__virtuemart_product_prices AS pp ON (oi.virtuemart_product_id = pp.virtuemart_product_id)'
            )
            ->leftJoin(
                '#__virtuemart_products AS p ON (oi.virtuemart_product_id = p.virtuemart_product_id)'
            )
            ->leftJoin(
                '#__virtuemart_orders AS o ON (oi.virtuemart_order_id = o.virtuemart_order_id)'
            );

        if (!empty($query_where_parts)) {
            $products_ordered_obj->where(implode(' AND ', $query_where_parts));
        }

        if (!empty($order_by)) {
            $products_ordered_obj->order($order_by);
        }

        if ((int)$this->group_by_product_id == 1) {
            $products_ordered_obj->select('
                oi.virtuemart_order_id AS order_id,
                oi.virtuemart_product_id AS product_id,
                oi.order_item_sku AS sku,
                SUM(oi.product_item_price) AS price,
                SUM(oi.product_quantity) AS quantity,
                pl.product_name AS name,
                pp.product_price AS current_price,
                p.product_gtin,
                oi.virtuemart_order_item_id AS id_order_product,
                p.published AS status_code
            ');
            $products_ordered_obj->group($db->quoteName('oi.virtuemart_product_id'));
        } else {
            $products_ordered_obj->select('
                oi.virtuemart_order_id AS order_id,
                oi.virtuemart_product_id AS product_id,
                pl.product_name AS name,
                oi.order_item_sku AS sku,
                oi.product_item_price AS price,
                pp.product_price AS current_price,
                p.product_gtin,
                o.order_status,
                oi.product_quantity AS quantity,
                os.order_status_name AS status,
                oi.virtuemart_order_item_id AS id_order_product,
                o.created_on AS date_add,
                p.published AS status_code
            ');

            $products_ordered_obj->leftJoin(
                '#__virtuemart_orderstates AS os ON (os.order_status_code = o.order_status)'
            );
        }

        if ($this->sort_by != 'price') {
            $db->setQuery($products_ordered_obj, ($this->page - 1) * $this->show, $this->show);
        } else {
            $db->setQuery($products_ordered_obj);
        }

        $products_ordered_res = $db->loadAssocList();

        foreach ($products_ordered_res as $product_ordered) {
            $product_ordered_info = $product_model->getProduct((int) $product_ordered['product_id'], true, true, false);

            $product_ordered['price'] = $this->convertPrice(
                $product_ordered_info->prices['basePrice'],
                $this->def_currency,
                $this->currency_code
            );

            // Get url of product image
            $product_image_id = $product_model
                ->getProduct($product_ordered['product_id'])
                ->virtuemart_media_id[0];
            $image_url = JURI::root() . VmModel::getModel('media')->getData($product_image_id)->file_url;

            if ($image_url && $image_url != JURI::root()) {
                $product_ordered['product_image'] = $image_url;
            }

            $products[] = $product_ordered;
        }

        if ($this->sort_by == 'price') {
            usort($products, array($this, 'sortItems'));
        }

        $product_from = ($this->page - 1) * $this->show;
        $product_to = $this->show + 1;
        $products_show = array_slice($products, $product_from, $product_to);

        for ($i = 0; $i < count($products_show); $i++) {
            $products_show[$i]['price'] = $this->formatPrice(
                $products_show[$i]['price'],
                $this->currency_code
            );
        }

        // Get count of ordered products
        $products_ordered_count_obj = $db->getQuery(true);
        if ((int)$this->group_by_product_id == 1) {
            $products_ordered_count_obj->select('
                COUNT(DISTINCT oi.virtuemart_product_id) AS count_prods,
                MAX(o.created_on) AS max_date,
                MIN(o.created_on) AS min_date
            ');
        } else {
            $products_ordered_count_obj->select('
                COUNT(oi.virtuemart_product_id) AS count_prods,
                MAX(o.created_on) AS max_date,
                MIN(o.created_on) AS min_date
            ');
        }
        $products_ordered_count_obj->from($db->quoteName('#__virtuemart_order_items', 'oi'));
        $products_ordered_count_obj->leftJoin(
            '#__virtuemart_orders AS o ON(oi.virtuemart_order_id = o.virtuemart_order_id)'
        );

        if (!empty($query_where_parts)) {
            $products_ordered_count_obj->where(implode(' AND ', $query_where_parts));
        }

        $db->setQuery($products_ordered_count_obj);
        $products_ordered_count = $db->loadAssoc();

        return array(
            'products_count' => (int)$products_ordered_count['count_prods'],
            'products'       => $products_show,
            'max_date'       => $products_ordered_count['max_date'],
            'min_date'       => $products_ordered_count['min_date'],
        );
    }

    private function getProductsInfo()
    {
        $this->product_id = (int)$this->product_id;

        if ($this->product_id < 1) {
            return false;
        }

        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query
            ->select($db->quoteName('virtuemart_product_id'))
            ->from($db->quoteName('#__virtuemart_products'))
            ->where($db->quoteName('virtuemart_product_id') . ' = ' . $this->product_id);
        $db->setQuery($query);
        $product_id = $db->loadResult();

        $product_obj = VmModel::getModel('product')->getProduct($this->product_id, true, true, false);

        if (!$product_id) {
            return false;
        }

        $images = $product_obj->virtuemart_media_id;
        $processed_image_ids = array();
        $count_images = count($images);
        $image_cover = array();
        $image_simple = array();

        for ($i = 0; $i < $count_images; $i++) {
            $image_id = $product_obj->virtuemart_media_id[$i];
            $images = VmModel::getModel('media')->getData($image_id);

            if (in_array($images->virtuemart_media_id, $processed_image_ids)) {
                continue;
            }

            $image = array(
                'small' => JURI::root() . $images->file_url,
                'large' => JURI::root() . $images->file_url
            );

            $db = JFactory::getDbo();
            $media_cover = $db->getQuery(true);
            $media_cover
                ->select($db->quoteName('ordering'))
                ->from($db->quoteName('#__virtuemart_product_medias'))
                ->where("virtuemart_media_id = '" . $product_obj->virtuemart_media_id[$i] . "' AND ordering = 1");
            $db->setQuery($media_cover);
            $media_cover = $db->loadResult();

            if ((int)$media_cover == 1) {
                $image_cover[] = $image;
            } else {
                $image_simple[] = $image;
            }

            $processed_image_ids[] = $images->virtuemart_media_id;
        }

        $query_ordered = $db->getQuery(true);
        $query_ordered
            ->select($db->quoteName('product_ordered'))
            ->from($db->quoteName('#__virtuemart_products'))
            ->where($db->quoteName('virtuemart_product_id') . ' = ' . $product_obj->virtuemart_product_id);
        $db->setQuery($query_ordered);
        $total_ordered = $db->loadResult();
        $total_ordered = null == $total_ordered ? 0 : $total_ordered;
        $price = $this->convertPrice(
            $product_obj->prices['basePrice'],
            $product_obj->prices['product_currency'],
            $this->currency_code
        );

        $product = array(
            'id_product'              => $product_obj->virtuemart_product_id,
            'name'                    => htmlspecialchars_decode($product_obj->product_name),
            'quantity'                => $product_obj->product_in_stock,
            'sku'                     => $product_obj->product_sku,
            'gtin'                    => (string)$product_obj->product_gtin,
            'active'                  => $product_obj->published,
            'images'                  => array_merge($image_cover, $image_simple),
            'total_ordered'           => $total_ordered,
            'price'                   => $price
        );
        $product['price'] = $this->formatPrice($product['price'], $this->currency_code);

        return $product;
    }

    private function getCustomPeriod($period = 0)
    {
        $custom_period = array('start_date' => '', 'end_date' => '');
        $format = 'm/d/Y';

        switch ($period) {
            case 0: //3 days
                $custom_period['start_date'] = date($format, mktime(0, 0, 0, date('m'), date('d') - 2, date('Y')));
                $custom_period['end_date'] = date($format, mktime(23, 59, 59, date('m'), date('d'), date('Y')));
                break;
            case 1: //7 days
                $custom_period['start_date'] = date($format, mktime(0, 0, 0, date('m'), date('d') - 6, date('Y')));
                $custom_period['end_date'] = date($format, mktime(23, 59, 59, date('m'), date('d'), date('Y')));
                break;
            case 2: //Prev week
                $custom_period['start_date'] = date(
                    $format,
                    mktime(0, 0, 0, date('m'), date('j') - 6, date('Y')) - ((date('N')) * 3600 * 24)
                );
                $custom_period['end_date'] = date(
                    $format,
                    mktime(23, 59, 59, date('m'), date('j'), date('Y')) - ((date('N')) * 3600 * 24)
                );
                break;
            case 3: //Prev month
                $custom_period['start_date'] = date($format, mktime(0, 0, 0, date('m') - 1, 1, date('Y')));
                $custom_period['end_date'] = date(
                    $format,
                    mktime(23, 59, 59, date('m'), date('d') - date('j'), date('Y'))
                );
                break;
            case 4: //This quarter
                $m = date('n');
                $start_m = 1;
                $end_m = 3;

                if ($m <= 3) {
                    $start_m = 1;
                    $end_m = 3;
                } elseif ($m >= 4 && $m <= 6) {
                    $start_m = 4;
                    $end_m = 6;
                } elseif ($m >= 7 && $m <= 9) {
                    $start_m = 7;
                    $end_m = 9;
                } elseif ($m >= 10) {
                    $start_m = 10;
                    $end_m = 12;
                }

                $custom_period['start_date'] = date($format, mktime(0, 0, 0, $start_m, 1, date('Y')));
                $custom_period['end_date'] = date($format, mktime(23, 59, 59, $end_m + 1, date(1) - 1, date('Y')));
                break;
            case 5: //This year
                $custom_period['start_date'] = date($format, mktime(0, 0, 0, date(1), date(1), date('Y')));
                $custom_period['end_date'] = date($format, mktime(23, 59, 59, date(1), date(1) - 1, date('Y') + 1));
                break;
            case 6: //Last year
                $custom_period['start_date'] = date($format, mktime(0, 0, 0, date(1), date(1), date('Y') - 1));
                $custom_period['end_date'] = date($format, mktime(23, 59, 59, date(1), date(1) - 1, date('Y')));
                break;
            case 8: //Previous quarter
                $m = date('n');
                $start_m = 1;
                $end_m = 3;

                if ($m <= 3) {
                    $start_m = 10;
                    $end_m = 12;
                } elseif ($m >= 4 && $m <= 6) {
                    $start_m = 1;
                    $end_m = 3;
                } elseif ($m >= 7 && $m <= 9) {
                    $start_m = 4;
                    $end_m = 6;
                } elseif ($m >= 10) {
                    $start_m = 7;
                    $end_m = 9;
                }

                $custom_period['start_date'] = date($format, mktime(0, 0, 0, $start_m, 1, date('Y')));
                $custom_period['end_date'] = date($format, mktime(23, 59, 59, $end_m + 1, date(1) - 1, date('Y')));
                break;
        }

        return $custom_period;
    }

    private function convertPrice($price, $from_id_currency, $to_id_currency)
    {
        if ($from_id_currency == $to_id_currency) {
            return $price;
        }

        $currency = CurrencyDisplay::getInstance($to_id_currency);
        return $currency->convertCurrencyTo($to_id_currency, $price, $to_id_currency == $this->def_currency);
    }

    private function formatPrice($price, $id_currency)
    {
        return CurrencyDisplay::getInstance($id_currency)->getFormattedCurrency($price);
    }

    private function runSelfTest()
    {
        $html = '<h2>Mobile Assistant Connector (v. '.JControllerLegacy::getInstance($this->module_name)->version.')</h2>
            <div style="margin-top: 15px; font-size: 13px;">Mobile Assistant Connector by
            <a href="http://emagicone.com" target="_blank" style="color: #15428B">eMagicOne</a></div>';

        die($html);
    }

    private function validateTypes(&$array, $names)
    {
        foreach ($names as $name => $type) {
            if (isset($array["$name"])) {
                switch ($type) {
                    case 'INT':
                        $array["$name"] = (int)$array["$name"];
                        break;
                    case 'FLOAT':
                        $array["$name"] = (float)$array["$name"];
                        break;
                    case 'STR':
                        $array["$name"] = str_replace(
                            array("\r", "\n"),
                            ' ',
                            addslashes(htmlspecialchars(trim(urldecode($array["$name"]))))
                        );
                        break;
                    case 'STR_HTML':
                        $array["$name"] = addslashes(trim(urldecode($array["$name"])));
                        break;
                    default:
                        $array["$name"] = '';
                }
            } else {
                $array["$name"] = '';
            }
        }

        $array_keys = array_keys($array);

        foreach ($array_keys as $key) {
            if (!isset($names[$key]) && $key != 'call_function' && $key != 'hash') {
                $array[$key] = '';
            }
        }

        return $array;
    }

    public function getIsset($key)
    {
        if (!isset($key) || empty($key) || !is_string($key)) {
            return false;
        }
        return isset($_POST[$key]) || isset($_GET[$key]);
    }

    private function getProductsDescr()
    {
        $this->product_id = (int)$this->product_id;
        $this->shop_id = (int)$this->shop_id;

        if ($this->product_id < 1) {
            return false;
        }

        $product_obj = VmModel::getModel('product')->getProduct($this->product_id, true, true, false);

        if (!$product_obj->id) {
            return false;
        }

        return array(
            'descr'       => $product_obj->product_desc,
            'short_descr' => $product_obj->product_s_desc
        );
    }

    private function getCustomers()
    {
        $customers = array();
        $order_by = array();
        $query_where_parts = array();

        if (!empty($this->customers_from)) {
            $query_where_parts[] = "u.registerDate >= '".date('Y-m-d H:i:s', strtotime($this->customers_from.' 00:00:00'))
                ."'";
        }

        if (!empty($this->customers_to)) {
            $query_where_parts[] = "u.registerDate <= '".date('Y-m-d H:i:s', strtotime($this->customers_to.' 23:59:59'))
                ."'";
        }

        if (!empty($this->search_val) && preg_match('/^\d+(?:,\d+)*$/', $this->search_val)) {
            $query_where_parts[] = 'u.id IN ('.JFilterInput::getInstance()->clean($this->search_val).')';
        } elseif (!empty($this->search_val)) {
            $query_where_parts[] = "(u.email LIKE '%" . JFilterInput::getInstance()->clean($this->search_val)
                . "%' OR u.name LIKE '%"
                . JFilterInput::getInstance()->clean($this->search_val)."%')";
        }

        if (!empty($this->cust_with_orders)) {
            $query_where_parts[] = 'tot.total_orders > 0';
        }

        if (empty($this->sort_by)) {
            $this->sort_by = 'id';
        }

        switch ($this->sort_by) {
            case 'id':
                $order_by = 'u.id '.$this->getSortDirection('DESC');
                break;
            case 'date':
                $order_by = 'u.registerDate '.$this->getSortDirection('DESC');
                break;
            case 'name':
                $order_by = 'u.name '.$this->getSortDirection('ASC');
                break;
            case 'qty':
                $order_by = 'total_orders '.$this->getSortDirection('DESC');
                break;
        }

        // Get customers
        $db = JFactory::getDbo();
        $customers_obj = $db->getQuery(true);

        $customers_obj
            ->select("
                u.id AS id_customer,
                u.name,
                u.email,
                u.registerDate AS date_add,
                IFNULL(tot.total_orders, 0) AS total_orders")
            ->from($db->quoteName('#__users', 'u'))
            ->join('LEFT OUTER','(SELECT COUNT(virtuemart_order_id) AS total_orders,
                 virtuemart_user_id
                FROM (#__virtuemart_orders)
                GROUP BY virtuemart_user_id) AS tot ON (tot.virtuemart_user_id = u.id)')
            ->leftJoin($db->quoteName('#__virtuemart_vmusers', 'vu') . ' ON('
                . $db->quoteName('vu.virtuemart_user_id') . ' = ' . $db->quoteName('u.id') . ')');

        if (!empty($query_where_parts)) {
            $customers_obj->where(implode(' AND ', $query_where_parts));
        }

        if (!empty($order_by)) {
            $customers_obj->order($order_by);
        }
        $db->setQuery($customers_obj, ($this->page - 1) * $this->show, $this->show);
        $customers_res = $db->loadAssocList();
        $customers_count = count($customers_res);
        $customer_ids = array();

        for ($i = 0; $i < $customers_count; $i++) {
            $date = explode(' ', $customers_res[$i]['date_add']);
            $customers_res[$i]['date_add'] = $date[0];
            $customers_res[$i]['full_name'] = $customers_res[$i]['name'];

            if (!in_array($customers_res[$i]['id_customer'], $customer_ids)) {
                $customers[] = $customers_res[$i];
                $customer_ids[] = $customers_res[$i]['id_customer'];
            }
        }

        // Get count of customers
        $db = JFactory::getDbo();
        $customers_count_obj = $db->getQuery(true);

        $customers_count_obj
            ->select('COUNT(u.id) AS count_custs,
                MAX(u.registerDate) AS max_date,
                MIN(u.registerDate) AS min_date')
            ->from($db->quoteName('#__users', 'u'))
            ->join('LEFT OUTER','(SELECT COUNT(virtuemart_order_id) AS total_orders, virtuemart_user_id 
              FROM #__virtuemart_orders GROUP BY virtuemart_user_id) AS tot ON tot.virtuemart_user_id = u.id')
            ->leftJoin($db->quoteName('#__virtuemart_vmusers', 'vu') . ' ON('
                . $db->quoteName('vu.virtuemart_user_id') . ' = ' . $db->quoteName('u.id') . ')')
            ->group($db->quoteName('u.id'));

        if (!empty($query_where_parts)) {
            $customers_count_obj->where(implode(' AND ', $query_where_parts));
        }
        $db->setQuery($customers_count_obj);
        $customers_count = count($db->loadAssocList());

        return array(
            'customers_count' => (int)$customers_count,
            'customers'       => $customers,
        );
    }

    private function getCustomersInfo()
    {
        $db = JFactory::getDbo();

        if ((int)$this->user_id < 1) {
            return false;
        }

        $customer = JUser::getInstance($this->user_id);

        if (!$customer->id) {
            return false;
        }

        if ((int)$this->only_items == 1) {
            return array('customer_orders' => $this->getCustomerItems());
        }

        // Get all customer addresses
        $addressesBT = VmModel::getModel('user')->getUserAddressList($this->user_id, 'BT');
        $addressesST = VmModel::getModel('user')->getUserAddressList($this->user_id, 'ST');
        $state = VmModel::getModel('state');
        $countSTadresses = count($addressesST);
        $billing_address = array();
        $shipping_address = array();

        if(!empty($addressesBT) || !empty($addressesST)) {
            for ($i = 0; $i < $countSTadresses; $i++) {
                $shipping_address[] = array(
                    'alias'         => $addressesST[$i]->address_type_name,
                    'first_name'    => $addressesST[$i]->first_name,
                    'last_name'     => $addressesST[$i]->last_name,
                    'company'       => $addressesST[$i]->company,
                    'address1'      => $addressesST[$i]->address_1,
                    'address2'      => $addressesST[$i]->address_2,
                    'postcode'      => $addressesST[$i]->zip,
                    'city'          => $addressesST[$i]->city,
                    'country'       => $addressesST[$i]->virtuemart_country_id,
                    'state'         => $state->getSingleState($addressesST[$i]->virtuemart_state_id)->state_name,
                    'phone'         => $addressesST[$i]->phone_1,
                    'phone_mobile'  => $addressesST[$i]->phone_2,
                );

                if (!empty($shipping_address[$i]['country'])) {
                    $ship_adresses_obj = $db->getQuery(true);
                    $ship_adresses_obj
                        ->select($db->quoteName('country_name'))
                        ->from($db->quoteName('#__virtuemart_countries'));
                    $ship_adresses_obj->where('virtuemart_country_id = ' . $shipping_address[$i]['country']);
                    $db->setQuery($ship_adresses_obj);
                    $shipping_address[$i]['country'] = $db->loadResult();
                }
            }

            $billing_address[] = array(
                'first_name'    => $addressesBT[0]->first_name,
                'last_name'     => $addressesBT[0]->last_name,
                'company'       => $addressesBT[0]->company,
                'address1'      => $addressesBT[0]->address_1,
                'address2'      => $addressesBT[0]->address_2,
                'postcode'      => $addressesBT[0]->zip,
                'city'          => $addressesBT[0]->city,
                'country'       => $addressesBT[0]->virtuemart_country_id,
                'state'         => $state->getSingleState($addressesBT[0]->virtuemart_state_id)->state_name,
                'phone'         => $addressesBT[0]->phone_1,
                'phone_mobile'  => $addressesBT[0]->phone_2,
            );

            if (!empty($billing_address[0]['country'])) {
                $bill_adresses_obj = $db->getQuery(true);
                $bill_adresses_obj
                    ->select($db->quoteName('country_name'))
                    ->from($db->quoteName('#__virtuemart_countries'));
                $bill_adresses_obj->where('virtuemart_country_id = ' . $billing_address[0]['country']);
                $db->setQuery($bill_adresses_obj);
                $billing_address[0]['country'] = $db->loadResult();
            }
        }


        // Get customer general information
        $customer_info = array(
            'id_customer'   => $customer->id,
            'full_name'     => $customer->name,
            'date_add'      => $customer->registerDate,
            'email'         => $customer->email,
        );

        // Get count of orders for customer
        $db = JFactory::getDbo();
        $customer_orders_count_obj = $db->getQuery(true);
        $customer_orders_count_obj
            ->select('
            o.virtuemart_order_id,
            o.order_total AS sum_ords, 
            o.user_currency_id
        ')
            ->from($db->quoteName('#__virtuemart_orders', 'o'))
            ->where('o.virtuemart_user_id = '.(int)$this->user_id);
        $db->setQuery($customer_orders_count_obj);
        $customer_orders = $db->loadAssocList();

        $customer_orders_count_res['sum_ords'] = 0;
        $customer_orders_count_res['count_ords'] = 0;
        foreach ($customer_orders as $order) {
            $order['sum_ords'] = $this->convertPrice(
                $order['sum_ords'],
                $order['user_currency_id'],
                $this->currency_code
            );

            $customer_orders_count_res['sum_ords'] += $order['sum_ords'];
            $customer_orders_count_res['count_ords'] += 1;
        }

        $customer_orders_count['sum_ords'] = $this->formatPrice(
            $customer_orders_count_res['sum_ords'],
            $this->currency_code
        );

        $customer_addresses['billing_address'] = $billing_address;
        $customer_addresses['shipping_address'] = $shipping_address;

        return array(
            'user_info'       => $customer_info,
            'addresses'       => $customer_addresses,
            'customer_orders' => $this->getCustomerItems(),
            'c_orders_count'  => $customer_orders_count_res['count_ords'],
            'sum_ords'        => $customer_orders_count['sum_ords']
        );
    }

    private function getCustomerItems()
    {
        $db = JFactory::getDbo();
        $customer_orders_obj = $db->getQuery(true);

        $customer_orders_obj
            ->select('
            o.virtuemart_order_id AS id_order,
            o.created_on AS date_add,
            o.order_total AS total_paid,
            o.order_currency AS id_currency,
            o.order_status AS ord_status_code,
            os.order_status_name AS ord_status,
            c.currency_code_3 AS iso_code,
            ui.first_name AS firstname,
            ui.last_name AS lastname,
            SUM(oi.product_quantity) AS pr_qty
        ')
            ->from($db->quoteName('#__virtuemart_orders', 'o'))
            ->leftJoin('#__virtuemart_currencies AS c ON (o.order_currency = c.virtuemart_currency_id)')
            ->leftJoin('#__virtuemart_order_items AS oi ON (oi.virtuemart_order_id = o.virtuemart_order_id)')
            ->leftJoin('#__virtuemart_order_userinfos AS ui ON (ui.virtuemart_order_id = o.virtuemart_order_id)')
            ->leftJoin('#__virtuemart_orderstates AS os ON (os.order_status_code = o.order_status)')
            ->where('o.virtuemart_user_id = '.(int)$this->user_id)
            ->group('id_order')
            ->order($db->quoteName('id_order') . ' DESC');

        $db->setQuery($customer_orders_obj, ($this->page - 1) * $this->show,  $this->show);
        $customer_orders = $db->loadAssocList();
        $count = count($customer_orders);

        for ($i = 0; $i < $count; $i++) {
            if ($this->currency_code != $customer_orders[$i]['id_currency']) {
                $customer_orders[$i]['total_paid'] = $this->convertPrice(
                    $customer_orders[$i]['total_paid'],
                    $customer_orders[$i]['id_currency'],
                    $this->currency_code
                );
            }

            $customer_orders[$i]['total_paid'] = $this->formatPrice(
                $customer_orders[$i]['total_paid'],
                $this->currency_code
            );
            unset($customer_orders[$i]['id_currency']);
//            $customer_orders[$i]['ord_status'] = $customer_orders[$i]['ord_status'];
            $customer_orders[$i]['ord_status'] = vmText::_($customer_orders[$i]['ord_status']);
        }

        return $customer_orders;
    }

    private function getOrders()
    {
        $query_where_parts = array();
        $orders = array();

        if (!empty($this->statuses)) {
            $query_where_parts[] = "o.order_status IN ('".JFilterInput::getInstance()->clean(str_replace('|', "','", $this->statuses))."')";
        }

        if (!empty($this->search_order_id) && preg_match('/^\d+(?:,\d+)*$/', $this->search_order_id)) {
            $query_where_parts[] = 'o.virtuemart_order_id IN ('.JFilterInput::getInstance()->clean($this->search_order_id).')';
        } elseif (!empty($this->search_order_id)) {
            $query_where_parts[] = "(CONCAT(ui.first_name, ' ', ui.last_name) LIKE '%".JFilterInput::getInstance()->clean($this->search_order_id)
                ."%' OR o.order_number LIKE '%".JFilterInput::getInstance()->clean($this->search_order_id)."%')";
        }

        if (!empty($this->orders_from)) {
            $query_where_parts[] = "o.created_on >= '".date('Y-m-d H:i:s', strtotime($this->orders_from.' 00:00:00'))
                . "'";
        }

        if (!empty($this->orders_to)) {
            $query_where_parts[] = "o.created_on <= '".date('Y-m-d H:i:s', strtotime($this->orders_to.' 23:59:59'))."'";
        }

        if (empty($this->sort_by)) {
            $this->sort_by = 'id';
        }

        switch ($this->sort_by) {
            case 'id':
                $order_by = 'o.virtuemart_order_id '.$this->getSortDirection('DESC');
                break;
            case 'date':
                $order_by = 'o.created_on '.$this->getSortDirection('DESC');
                break;
            case 'name':
                $order_by = 'customer '.$this->getSortDirection('ASC');
                break;
            case 'total':
                $order_by = 'o.order_total '.$this->getSortDirection('DESC');
                break;
            case 'qty':
                $order_by = 'count_prods '.$this->getSortDirection('DESC');
                break;
            default:
                $order_by = '';
                break;
        }

        // Get orders
        $db = JFactory::getDbo();
        $orders_obj = $db->getQuery(true);

        $orders_obj
            ->select(
                "o.virtuemart_order_id AS id_order,
            o.created_on AS date_add,
            o.order_total AS total_paid,
            o.order_currency AS id_currency,
            o.order_status AS status_code,
            c.currency_code_3 AS iso_code,
            CONCAT(ui.first_name, ' ', ui.last_name) AS customer,
            os.order_status_name AS ord_status,
            (SELECT SUM(od.product_quantity) FROM `#__virtuemart_order_items` AS od 
            WHERE od.virtuemart_order_id = o.virtuemart_order_id) AS count_prods"
            )
            ->from($db->quoteName('#__virtuemart_orders', 'o'))
            ->leftJoin('#__virtuemart_currencies AS c ON (o.order_currency = c.virtuemart_currency_id)')
            ->leftJoin('#__virtuemart_order_userinfos AS ui ON (o.virtuemart_order_id = ui.virtuemart_order_id)')
//            ->leftJoin('#__virtuemart_order_userinfos AS ui ON (o.virtuemart_user_id = ui.virtuemart_user_id)')
            ->leftJoin('#__virtuemart_orderstates AS os ON (o.order_status = os.order_status_code)');
//            ->where($db->quoteName('ui.virtuemart_user_id') . ' = ' . $db->quoteName('o.virtuemart_user_id'));
//            ->where($db->quoteName('ui.virtuemart_order_id') . ' = ' . $db->quoteName('o.virtuemart_order_id'));

        if (!empty($query_where_parts)) {
            $orders_obj->where(implode(' AND ', $query_where_parts));
        }

        $orders_obj->group('id_order');
        $orders_obj->order($order_by);
        $db->setQuery($orders_obj, ($this->page - 1) * $this->show, $this->show);
        $orders_res = $db->loadAssocList();

        // Convert price data
        foreach ($orders_res as $order) {
            $order['total_paid'] = $this->convertPrice($order['total_paid'], $order['id_currency'], $this->currency_code);
            $order['ord_status'] = VmText::_($order['ord_status']);
            $order['total_paid'] = $this->formatPrice($order['total_paid'], $this->currency_code);
            unset($order['id_currency']);
            $orders[] = $order;
        }

        // Get orders statistics
        $db = JFactory::getDbo();
        $orders_stats_obj = $db->getQuery(true);

        $orders_stats_obj
            ->select('
            o.virtuemart_order_id AS count_ords,
            o.order_total AS total_paid,
            o.user_currency_id
        ')
            ->from($db->quoteName('#__virtuemart_orders', 'o'))
            ->leftJoin('#__virtuemart_currencies AS c ON (o.order_currency = c.virtuemart_currency_id)')
            ->leftJoin('#__virtuemart_order_userinfos AS ui ON (o.virtuemart_order_id = ui.virtuemart_order_id)')
            ->where('ui.address_type = "BT"');

        if (!empty($query_where_parts)) {
            $orders_stats_obj->where(implode(' AND ', $query_where_parts));
        }

        $db->setQuery($orders_stats_obj);
        $orders_stats = $db->loadAssocList();

        $orders_total = 0;
        $orders_count = 0;
        foreach ($orders_stats as $orders_info) {
            $orders_info['total_paid'] = $this->convertPrice($orders_info['total_paid'], $orders_info['user_currency_id'], $this->currency_code);
            $orders_total += $orders_info['total_paid'];
            $orders_count += 1;
        }

        $orders_total = $this->formatPrice($orders_total, $this->currency_code);

        // Get all order statuses
        $orders_status = null;
        if ((int)$this->get_statuses == 1) {
            $orders_status = $this->getOrdersStatuses();
        }

        return array(
            'orders'        => $orders,
            'orders_count'  => $orders_count,
            'orders_total'  => $orders_total,
            'orders_status' => $orders_status,
        );
    }

    private function getOrdersInfo()
    {
        if ((int)$this->order_id < 1) {
            return false;
        }

        $order_obj = VmModel::getModel('orders')->getOrder($this->order_id);

        if (empty($order_obj['details'])) {
            return false;
        }

        if ((int)$this->only_items == 1) {
            return array('order_products' => $this->getOrderItems());
        }

        $currency_obj = VmModel::getModel('currency')->getCurrency($order_obj['details']['BT']->order_currency);
        $order_obj_bt = $order_obj['details']['BT'];

        if ($order_obj['details']['has_ST']) {
            $order_obj_st = $order_obj['details']['ST'];
        } else {
            $order_obj_st = $order_obj['details']['BT'];
        }

        $db = JFactory::getDbo();
        $order_address_details = $db->getQuery(true);
        $order_address_details
            ->select('c.country_name, s.state_name, oui.address_type')
            ->from($db->quoteName('#__virtuemart_order_userinfos', 'oui'))
            ->leftJoin('#__virtuemart_countries AS c ON (c.virtuemart_country_id = oui.virtuemart_country_id)')
            ->leftJoin('#__virtuemart_states AS s ON (s.virtuemart_state_id = oui.virtuemart_state_id)')
            ->where('oui.virtuemart_order_id = ' . $order_obj_bt->virtuemart_order_id);
        $db->setQuery($order_address_details);
        $order_address_details_res = $db->loadAssoc();

        $order_bt_state = '';
        $order_bt_country = '';
        if ($order_address_details_res['address_type'] == "ST") {
            $order_st_state = $order_address_details_res['state_name'];
            $order_st_country = $order_address_details_res['country_name'];
        } else {
            $order_bt_state = $order_address_details_res['state_name'];
            $order_bt_country = $order_address_details_res['country_name'];
            $order_st_state = $order_bt_state;
            $order_st_country = $order_bt_country;
        }

        $order_payment_method = VmModel::getModel('paymentmethod')->getPayment($order_obj_bt->virtuemart_paymentmethod_id)->payment_name;
        $order_info = array(
            'id_order'          => $order_obj_bt->virtuemart_order_id,
            'date_add'          => $order_obj_bt->created_on,
            'id_customer'       => $order_obj_bt->virtuemart_user_id,
            'id_currency'       => $order_obj_bt->order_currency,
            'total_paid'        => $order_obj_bt->order_total,
            'total_discounts'   => $order_obj_bt->order_discountAmount,
            'total_shipping'    => $order_obj_bt->order_shipment,
            'status_code'       => $order_obj_bt->order_status,
            'status'            => !empty(VmText::_($order_obj_bt->order_status_name))
                ? VmText::_($order_obj_bt->order_status_name)
                : '',
            'iso_code'          => $currency_obj->currency_code_3,
            'customer'          => $order_obj_bt->first_name . ' ' . $order_obj_bt->last_name,
            'email'             => $order_obj_bt->email,
            'd_name'            => $order_obj_st->first_name . ' ' . $order_obj_st->last_name,
            'd_company'         => $order_obj_st->company,
            'd_address1'        => $order_obj_st->address_1,
            'd_address2'        => $order_obj_st->address_2,
            'd_city'            => $order_obj_st->city,
            'd_state'           => $order_st_state,
            'd_postcode'        => $order_obj_st->zip,
            'd_country'         => $order_st_country,
            'd_phone'           => $order_obj_st->phone_2,
            'd_phone_mobile'    => $order_obj_st->phone_1,
            'i_name'            => $order_obj_bt->first_name . ' ' . $order_obj_bt->last_name,
            'i_company'         => $order_obj_bt->company,
            'i_address1'        => $order_obj_bt->address_1,
            'i_address2'        => $order_obj_bt->address_2,
            'i_city'            => $order_obj_bt->city,
            'i_state'           => $order_bt_state,
            'i_postcode'        => $order_obj_bt->zip,
            'i_country'         => $order_bt_country,
            'i_phone'           => $order_obj_bt->phone_2,
            'i_phone_mobile'    => $order_obj_bt->phone_1,
            'p_method'          => $order_payment_method,
        );

        // Convert and format prices
        $elements_to_format = array(
            'total_paid',
            'total_discounts',
            'total_shipping',
        );
        $count = count($elements_to_format);

        for ($i = 0; $i < $count; $i++) {
            if ($this->currency_code != $order_obj_bt->order_currency) {
                $order_info[$elements_to_format[$i]] = $this->convertPrice(
                    $order_info[$elements_to_format[$i]],
                    $order_obj_bt->order_currency,
                    $this->currency_code
                );
            }

            $order_info[$elements_to_format[$i]] = $this->formatPrice(
                $order_info[$elements_to_format[$i]],
                $this->currency_code
            );
        }

        // Get number of products in order
        $order_products_count = count($order_obj['items']);
        $carrier_obj = VmModel::getModel('shipmentmethod')->getShipment($order_obj_bt->virtuemart_shipmentmethod_id);

        $order_carrier = array(
            'date_add'        => $order_obj_bt->created_on,
            'carrier_name'    => $carrier_obj->shipment_name,
            'shipping_cost'   => $order_obj_st->order_shipment,
            'id_currency'     => $order_obj_st->order_currency
        );

        $order_carrier['shipping_cost'] = $this->convertPrice(
            $order_carrier['shipping_cost'],
            $order_obj_st->order_currency,
            $this->currency_code
        );

        $order_carrier['shipping_cost'] = $this->formatPrice(
            $order_carrier['shipping_cost'],
            $order_obj_st->order_currency
        );

        return array(
            'order_info'       => $order_info,
            'order_products'   => $this->getOrderItems(),
            'o_products_count' => (int)$order_products_count,
            'order_tracking'   => array($order_carrier)
        );
    }

    private function getOrderItems()
    {
        $order_products_obj = VmModel::getModel('orders')->getOrder($this->order_id);
        $order_products_bt = $order_products_obj['details']['BT'];
        $count_products = count($order_products_obj['items']);
        $order_products = array();
        $currency = CurrencyDisplay::getInstance($this->currency_code);

        // Convert and form price data
        for ($i = 0; $i < $count_products; $i++) {
            $order_item = $order_products_obj['items'][$i];
            $order_products[$i] = array(
                'id_order_product'  => $order_item->virtuemart_order_item_id,
                'id_order'          => $order_products_bt->virtuemart_order_id,
                'product_id'        => $order_item->virtuemart_product_id,
                'product_name'      => $order_item->order_item_name,
                'sku'               => $order_item->order_item_sku,
                'product_price'     => $order_item->product_item_price,
                'product_quantity'  => $order_item->product_quantity,
                'wholesale_price'   => $order_item->product_discountedPriceWithoutTax,
                'id_currency'       => $order_products_bt->order_currency,
                'status_code'       => $order_item->published
            );

            $order_products[$i]['product_price'] = $this->convertPrice(
                $order_products_obj['items'][$i]->product_item_price,
                $order_products_obj['items'][$i]->prices['product_currency'],
                $this->currency_code
            );
            /*$order_products[$i]['product_price'] = $currency->convertCurrencyTo(
                $this->currency_code,
                $order_products_obj['items'][$i]->product_item_price,
                $this->currency_code == $this->def_currency);*/

            $order_products[$i]['wholesale_price'] = $this->convertPrice(
                $order_products_bt->order_salesPrice,
                $order_products_obj['items'][$i]->prices['product_currency'],
                $this->currency_code
            );
            /*$order_products[$i]['wholesale_price'] = $currency->convertCurrencyTo(
                $this->currency_code,
                $order_products_bt->order_salesPrice,
                $this->currency_code == $this->def_currency);*/

            $order_products[$i]['product_price'] = $this->formatPrice(
                $order_products[$i]['product_price'],
                $this->currency_code
            );

            $order_products[$i]['wholesale_price'] = $this->formatPrice(
                $order_products[$i]['wholesale_price'],
                $this->currency_code
            );

            // Get url of product image
            $image_media_id = $order_products_obj['items'][$i]->virtuemart_media_id[0];
            $images = VmModel::getModel('media')->getData($image_media_id);
            $image_url = JURI::root() . $images->file_url;

            if ($image_url && $image_url != JURI::root()) {
                $order_products[$i]['product_image'] = $image_url;
            }
        }

        return $order_products;
    }

    private function setOrderAction()
    {
        $this->order_id = (int)$this->order_id;

        if ($this->order_id < 1) {
            return array('error' => 'Order id cannot be empty!');
        }

        if (empty($this->action)) {
            return array('error' => 'Action is not set!');
        }
        $order = VmModel::getModel('orders')->getOrder($this->order_id);

        if (empty($order['details'])) {
            return array('error' => 'No order was found!');
        }

        $result = array('success' => false);

        switch ($this->action) {
            case 'change_status':
                if (isset($this->new_status)) {
                    $result = $this->changeStatusOrder();
                } else {
                    $result = array('error' => 'New order status is not set!');
                }

                break;
        }

        return $result;
    }

    private function changeStatusOrder()
    {
        $this->order_id = (int)$this->order_id;

        if ($this->order_id < 1) {
            return false;
        }

        try {
            $modelOrder = VmModel::getModel('orders');
            $modelOrder->updateStatusForOneOrder(
                $this->order_id,
                array(
                    'order_status' => JRequest::getVar('new_status'),
                    'comments' => '',
                    'customer_notified' => ''
                )
            );
        } catch (Exception $e) {
            JLog::add(JText::_('change_status_order: exception = '.$e->getMessage()), JLog::WARNING, 'com_mobileassistantconnector');
            return array('error' => $e->getMessage());
        }

        return array('success' => 'true');
    }

    private function getCarriers()
    {
        $carriers_ret = array();
        $carries = VmModel::getModel('shipmentmethod')->getShipments();

        foreach ($carries as $key => $carrier) {
            $carriers_ret[$key]['id_carrier'] = $carrier->virtuemart_shipmentmethod_id;
            $carriers_ret[$key]['name'] = $carrier->shipment_name;
        }

        return $carriers_ret;
    }

    private function getAbandonedCarts()
    {
        $db = JFactory::getDbo();
        $cart_total_res    = array();
        $carts             = array();
        $customfield_model = VmModel::getModel('customfields');
        $currency = CurrencyDisplay::getInstance($this->currency_code);

        if (!empty($this->search_carts) && preg_match('/^\d+(?:,\d+)*$/', $this->search_carts)) {
            $query_where_parts[] = 'c.virtuemart_cart_id IN ('.JFilterInput::getInstance()->clean($this->search_carts).')';
        } elseif (!empty($this->search_carts)) {
            $query_where_parts[] = "( COALESCE(CONCAT(ui.first_name, ' ', ui.last_name), 'Guest not registered') LIKE
                '%".JFilterInput::getInstance()->clean($this->search_carts)."%')";
        }

        if (!empty($this->carts_from)) {
            $query_where_parts[] = "(c.created_on <> '0000-00-00 00:00:00' AND c.created_on >= '"
                . date('Y-m-d H:i:s', strtotime($this->carts_from . ' 00:00:00'))
                . "' OR c.created_on = '0000-00-00 00:00:00' AND c.modified_on >= '"
                . date('Y-m-d H:i:s', strtotime($this->carts_from . ' 00:00:00')) . "')";
            $query_where_parts_for_count[] = "(created_on <> '0000-00-00 00:00:00' AND created_on >= '"
                . date('Y-m-d H:i:s', strtotime($this->carts_from . ' 00:00:00'))
                . "' OR created_on = '0000-00-00 00:00:00' AND modified_on >= '"
                . date('Y-m-d H:i:s', strtotime($this->carts_from . ' 00:00:00')) . "')";
        }

        if (!empty($this->carts_to)) {
            $query_where_parts[] = "(c.created_on <> '0000-00-00 00:00:00' AND c.created_on <= '"
                . date('Y-m-d H:i:s', strtotime($this->carts_to . ' 23:59:59'))
                . "' OR c.created_on = '0000-00-00 00:00:00' AND c.modified_on <= '"
                . date('Y-m-d H:i:s', strtotime($this->carts_to . ' 23:59:59')) . "')";
            $query_where_parts_for_count[] = "(created_on <> '0000-00-00 00:00:00' AND created_on <= '"
                . date('Y-m-d H:i:s', strtotime($this->carts_to . ' 23:59:59'))
                . "' OR created_on = '0000-00-00 00:00:00' AND modified_on <= '"
                . date('Y-m-d H:i:s', strtotime($this->carts_to . ' 23:59:59')) . "')";
        }

        if (empty($this->sort_by)) {
            $this->sort_by = 'id';
        }

        switch ($this->sort_by) {
            case 'id':
                $order_by = 'c.virtuemart_cart_id ' . $this->getSortDirection('DESC');
                break;
            case 'date':
                $order_by = 'c.created_on ' . $this->getSortDirection('DESC');
                break;
            case 'name':
                $order_by = 'customer ' . $this->getSortDirection('ASC');
                break;
            case 'total':
                $this->sort_by = 'cart_total_row';
                break;
            case 'qty':
                $this->sort_by = 'cart_count_products';
                break;
            default:
                $order_by = '';
                break;
        }

        // Get abandoned carts
        $carts_obj = $db->getQuery(true);
        $carts_obj->select("
            c.virtuemart_cart_id AS id_cart,
            c.created_on,
            c.modified_on,
            c.virtuemart_user_id AS id_customer,
            u.email,
            COALESCE(CONCAT(ui.first_name, ' ', ui.last_name), 'Guest not registered') AS customer
        ")
            ->from($db->quoteName('#__virtuemart_carts', 'c'))
            ->leftJoin('#__virtuemart_userinfos AS ui ON (c.virtuemart_user_id = ui.virtuemart_user_id)')
            ->leftJoin('#__users AS u ON(c.virtuemart_user_id = u.id)')
            ->group('c.virtuemart_cart_id');

        if (!empty($order_by)) {
            $carts_obj->order($order_by);
        }

        if (!empty($query_where_parts)) {
            $carts_obj->where(implode(' AND ', $query_where_parts));
        }

        if ($this->sort_by != 'cart_total_row' && $this->sort_by != 'cart_count_products') {
            $db->setQuery($carts_obj, ($this->page - 1) * $this->show, $this->show);
        } else {
            $db->setQuery($carts_obj);
        }

        $cart_object_result = $db->loadAssocList();
        $model = new VmModel();
        $cart_data = $model->getTable('carts');
        $cart_total_res['total_sum'] = '';
        $count_cart_result = count($cart_object_result);

        for ($k = 0; $k < $count_cart_result; $k++) {
            $item_sum_price = 0;
            $cart_data->load($cart_object_result[$k]['id_cart'], 'virtuemart_cart_id');
            $cartData = $cart_data->loadFieldValues();

            if (!is_array($cartData) || empty($cartData['cartData'])) {
                continue;
            }

            $cartData['cartData'] = (object)json_decode($cartData['cartData'], true);
            $cart_object_result[$k]['id_currency'] = $cartData['cartData']->pricesCurrency;
            $carrier_item = VmModel::getModel('shipmentmethod')
                ->getShipment($cartData['cartData']->virtuemart_shipmentmethod_id);
            $cart_object_result[$k]['carrier_name'] = $carrier_item->shipment_name;
            $cart_object_result[$k]['cart_count_products'] = count($cartData['cartData']->cartProductsData);

            //Get cart item data
            for ($i = 0; $i < $cart_object_result[$k]['cart_count_products']; $i++) {
                $cart_product_data = $cartData['cartData']->cartProductsData[$i];
                $item_id = $cartData['cartData']->cartProductsData[$i]['virtuemart_product_id'];
                $item_price = VmModel::getModel('product')->getProduct($item_id)->prices['product_price'];
                $item_price = $this->convertPrice(
                    $item_price,
                    VmModel::getModel('product')->getProduct($item_id)->prices['product_currency'],
                    $this->currency_code
                );

                $customfield_id = '';
                $customfield_price = 0;

                // Get product customfields value
                foreach ($cart_product_data['customProductData'] as $custom_field) {
                    if (is_array($custom_field)) {
                        foreach ($custom_field as $key => $value) {
                            $customfield_id = $key;
                        }
                    } else {
                        $customfield_id = $custom_field;
                    }

                    $customfield_price += $customfield_model
                        ->getCustomEmbeddedProductCustomField($customfield_id)
                        ->customfield_price;
                }

                if (null != $item_price) {
                    $item_sum_price += $cart_product_data['quantity'] * ($item_price + $customfield_price);
                }
            }

            $cart_object_result[$k]['cart_total_row'] = $item_sum_price;
            $cart_object_result[$k]['cart_total'] = $this->formatPrice($item_sum_price, $this->currency_code);
            $cart_object_result[$k]['date_add'] =
                $cart_object_result[$k]['created_on'] > 0 ?
                    $cart_object_result[$k]['created_on']
                    : $cart_object_result[$k]['modified_on'];
            unset($cart_object_result[$k]['created_on']);
            unset($cart_object_result[$k]['modified_on']);
            $carts[] = $cart_object_result[$k];

            $cart_total_res['total_sum'] += $cart_object_result[$k]['cart_total_row'];
        }

        /*$cart_total_res['total_sum'] = $currency->convertCurrencyTo(
            $this->currency_code,
            $cart_total_res['total_sum'],
            $this->currency_code == $this->def_currency
        );*/

        if ($this->sort_by == 'cart_total_row' || $this->sort_by == 'cart_count_products') {
            usort($carts, array($this, 'sortItems'));
        }

        $carts_count = $db->getQuery(true);
        $carts_count
            ->select('virtuemart_cart_id')
            ->from('#__virtuemart_carts');

        if (!empty($query_where_parts_for_count)){
            $carts_count->where(implode(' AND ', $query_where_parts_for_count));
        }

        $db->setQuery($carts_count);
        $carts_count = $db->loadAssocList();

        $cart_total_res['total_count'] = count($carts_count);
        $cart_total_res['total_sum'] = $this->formatPrice(
            $cart_total_res['total_sum'],
            $this->currency_code
        );
        $cart_from = ($this->page - 1) * $this->show;
        $cart_to = $this->show + 1;
        $carts = array_slice($carts, $cart_from, $cart_to);

        return array (
            'abandoned_carts'       => $carts,
            'abandoned_carts_count' => $cart_total_res['total_count'],
            'abandoned_carts_total' => $cart_total_res['total_sum']
        );
    }

    private function getAbandonedCartInfo()
    {
        $this->cart_id = (int)$this->cart_id;

        if ($this->cart_id < 1) {
            return false;
        }

        $this->only_items = (int)$this->only_items;

        if ($this->only_items != 1) {

            // Get cart information
            $db = JFactory::getDbo();
            $cart_info_obj = $db->getQuery(true);
            $cart_info_obj->select("
                    c.virtuemart_cart_id AS id_cart,
                    c.virtuemart_user_id AS id_customer,
                    u.registerDate AS account_registered, 
                    c.modified_on AS date_add
                ")
                ->from($db->quoteName('#__virtuemart_carts', 'c'))
                ->leftJoin('#__users AS u ON(c.virtuemart_user_id = u.id)')
                ->where('c.virtuemart_cart_id = ' . (int)$this->cart_id);
            $db->setQuery($cart_info_obj);
            $cart_info = $db->loadAssoc();

            if (!$cart_info) {
                return false;
            }
        }

        $model = new VmModel();

        $carts = $model->getTable('carts');
        $carts->load((int)$this->cart_id, 'virtuemart_cart_id');
        $cartData = $carts->loadFieldValues();
        $items_sum_price = 0;

        if (is_array($cartData) && !empty($cartData['cartData'])) {
            $cartData['cartData'] = (object)json_decode($cartData['cartData'], true);
            $cart_products_count = count($cartData['cartData']->cartProductsData);

            for ($i = 0; $i < $cart_products_count; $i++) {
                $item_image_id = '';
                $cart_product_data = $cartData['cartData']->cartProductsData[$i];
                $item_id = $cart_product_data['virtuemart_product_id'];
                $item_model = VmModel::getModel('product')->getProduct($item_id);
                $item_media_model = VmModel::getModel('media');
                $customfield_model = VmModel::getModel('customfields');

                $cart_items[$i]['id_product'] = $item_id;
                $cart_items[$i]['sku'] = $item_model->product_sku;
                $cart_items[$i]['product_name'] = $item_model->product_name;
                $cart_items[$i]['product_quantity'] = $cartData['cartData']->cartProductsData[$i]['quantity'];
                $cart_items[$i]['id_currency'] = $item_model->prices['product_currency'];
				$cart_items[$i]['status_code'] = (int)$item_model->published;
                /*$cart_info['customer'] = $cartData['cartData']->BT['last_name']
                    . ' ' . $cartData['cartData']->BT['first_name'];*/
                $cart_info['date_add'] = $cartData['modified_on'];

                if (!empty($cartData['cartData']->BT['phone_1'])){
                    $cart_info['phone'] = $cartData['cartData']->BT['phone_1'];
                } else {
                    $cart_info['phone'] = $cartData['cartData']->BT['phone_2'];
                }

                if ($item_model->virtuemart_media_id) {
                    $item_image_id = $item_model->virtuemart_media_id[0];
                }

                $cart_items[$i]['product_image'] = JURI::root() . $item_media_model->getData($item_image_id)->file_url;
                $customfield_id = '';
                $customfield_price = 0;
                $customfield_combination = array();

                foreach ($cart_product_data['customProductData'] as $custom_field) {
                    if (is_array($custom_field)) {
                        foreach ($custom_field as $key => $value){
                            $customfield_id = $key;
                        }
                    } else {
                        $customfield_id = $custom_field;
                    }

                    $customfield_title = $customfield_model->getCustomEmbeddedProductCustomField($customfield_id)->custom_title;
                    $customfield_value = $customfield_model->getCustomEmbeddedProductCustomField($customfield_id)->customfield_value;
                    $customfield_price += $customfield_model->getCustomEmbeddedProductCustomField($customfield_id)->customfield_price;
                    $customfield_combination[] = $customfield_title . ' : ' . $customfield_value;
                }

                $cart_items[$i]['combination'] = !empty($customfield_combination) ? implode('\n ', $customfield_combination) : '';
                $item_price = $item_model->prices['product_price'] + $customfield_price;
                $cart_items[$i]['product_price'] = $this->convertPrice(
                    $item_price,
                    $item_model->prices['product_currency'],
                    $this->currency_code
                );
                
                $cart_items[$i]['product_price'] = $this->formatPrice(
                    $cart_items[$i]['product_price'],
                    $this->currency_code
                );

                if ($this->only_items != 1) {
                    $items_sum_price += $cartData['cartData']->cartProductsData[$i]['quantity'] * $item_price;
                }
            }

            if ($this->only_items != 1) {
                $cart_info['id_currency'] = $cartData['cartData']->pricesCurrency;
                $carrier_id = $cartData['cartData']->virtuemart_shipmentmethod_id;
                $cart_info['carrier_name'] = VmModel::getModel('shipmentmethod')->getShipment($carrier_id)->shipment_name;
                $cart_info['cart_total'] = $items_sum_price;
            }
        }

        if ($cart_info['id_customer'] >= 0) {
            $cart_info['customer'] = $cartData['cartData']->BT['first_name']
                . ' ' . $cartData['cartData']->BT['last_name'];
            $cart_info['email'] = $cartData['cartData']->BT['email'];
        } elseif (trim($cart_info['customer']) == '') {
            $cart_info['customer'] = self::GUEST;
        }

        // Convert and format price data
        $cart_info['cart_total'] = $this->convertPrice(
            $cart_info['cart_total'],
            $cart_info['id_currency'],
            $this->currency_code
        );
        /*$cart_info['cart_total'] = $currency->convertCurrencyTo(
            $this->currency_code,
            $cart_info['cart_total'],
            $this->currency_code == $this->def_currency
        );*/

        $cart_info['cart_total'] = $this->formatPrice($cart_info['cart_total'], $this->currency_code);
        $cart_products_count = count($cartData['cartData']->cartProductsData);

        if ($this->only_items == 1) {
            return array(
                'cart_products'       => $cart_items,
            );
        } else {
            return array(
                'cart_info'           => $cart_info,
                'cart_products'       => $cart_items,
                'cart_products_count' => (int)$cart_products_count,
            );
        }
    }

    private function sortItems($a, $b)
    {
        if ($a[$this->sort_by] == $b[$this->sort_by]) {
            return 0;
        }

        $order_by = $this->getSortDirection($this->order_by);

        if ($order_by == "asc") {
            return ($a[$this->sort_by] < $b[$this->sort_by]) ? -1 : 1;
        } elseif ($order_by == "desc") {
            return ($b[$this->sort_by] < $a[$this->sort_by]) ? -1 : 1;
        }
    }

    private function getOrderPdf()
    {
        $this->order_id =(int)$this->order_id;

        if ($this->order_id < 1) {
            return 'Order id cannot be empty!';
        }

        $orderModel = VmModel::getModel('orders');
        $order = $orderModel->getOrder($this->order_id);

        if (empty($order['details'])) {
            return 'No order was found!';
        }

        $controller = new VirtueMartControllerInvoice();
        $invoicePdfPath = $controller->getInvoicePDF($order);
        header ('Content-Type: application/pdf');

        return $invoicePdfPath ? file_get_contents($invoicePdfPath) : 'No invoice was found!';
    }

    private function generateOutput($data)
    {
        JFactory::getDocument()->setMimeEncoding( 'application/json' );
        $add_bridge_version = false;

        if (
        in_array(
            $this->call_function,
            array('test_config', 'get_store_title', 'get_store_stats', 'get_data_graphs', 'get_version')
        )
        ) {
            if (is_array($data) && $data != 'auth_error' && $data != 'connection_error' && $data != 'old_bridge') {
                $add_bridge_version = true;
            }
        }

        if (!is_array($data)) {
            $data = array($data);
        } else {
            $data['module_response'] = '1';
        }

        if (is_array($data)) {
            array_walk_recursive($data, array($this, 'resetNull'));
        }

        if ($add_bridge_version) {
            $data['module_version'] = self::MODULE_VERSION;
        }

        echo json_encode($data);

        header('Content-Type: text/javascript;charset=utf-8');
        JFactory::getApplication()->close();
        die($data);
    }

    private function resetNull(&$item, $key)
    {
        if (empty($item) && $item !== 0) {
            $item = '';
        }

        $item = trim($item);
    }

    private function getAccountIdByEmail($account_email)
    {
        $db = JFactory::getDbo();
        if (empty($account_email)) {
            return false;
        }
        $account_id = $db->getQuery(true);
        $account_id
            ->select($db->quoteName('id'))
            ->from($db->quoteName('#__mobassistantconnector_accounts'))
            ->where($db->quoteName('account_email') . " = '" . JFilterInput::getInstance()->clean($account_email) . "'");
        $db->setQuery($account_id);
        $account_id = $db->loadResult();

        if (!$account_id) {
            $insert = $db->getQuery(true);
            $insert
                ->insert($db->quoteName('#__mobassistantconnector_accounts'))
                ->columns($db->quoteName(array('account_email', 'status')))
                ->values(implode(',',array("'".JFilterInput::getInstance()->clean($account_email)."'",1)));
            $insert = $db->setQuery($insert);
            $db->execute();

            if ($insert) {
                $account_id = $db->getQuery(true);
                $account_id
                    ->select('id')
                    ->from($db->quoteName('#__mobassistantconnector_accounts'))
                    ->where($db->quoteName('account_email') . " = '" . JFilterInput::getInstance()->clean($account_email)) . "' ";
                $db->setQuery($account_id);
                $account_id = $db->loadAssocList();
            }
        }

        return $account_id;
    }

    private function getQrCode($hash)
    {
        $db = JFactory::getDbo();

        $result = 'SELECT `username`, `password` FROM `'
            ."#__mobassistantconnector_users` WHERE `qr_code_hash` = '".JFilterInput::getInstance()->clean($hash)."' AND `status` = 1";

        $db->setQuery($result);
        $result = $db->loadAssoc();

        if ($result) {
            $data_to_qr = MobileassistantconnectorHelperQrcode::getDataToQrCode(
                JUri::root(),
                $result['username'],
                $result['password']
            );

            include JPATH_COMPONENT_ADMINISTRATOR . '/views/qr_code.phtml';

            echo '<script type="text/javascript">
                (function() {
                    var qrcode = new QRCode(document.getElementById("mobileassistantconnector_qrcode_img"), {
                        width : 300,
                        height : 300
                    });
                    qrcode.makeCode("' . $data_to_qr . '");
            })();
            </script>';

        } else {
            $this->generateOutput('auth_error');
        }
        die();
    }

    public function getVersion()
    {
        $session_key = '';

        if ($this->hash) {
            $user_data = MobileassistantconnectorAccess::checkAuth($this->hash, $this->file_logger);

            if ($user_data) {
                if ($this->session_key) {
                    if (MobileassistantconnectorAccess::checkSessionKey($this->session_key, $user_data['user_id'])) {
                        $session_key = $this->session_key;
                    } else {
                        $session_key = MobileassistantconnectorAccess::getSessionKey($this->hash, $user_data['user_id']);
                    }
                } else {
                    $session_key = MobileassistantconnectorAccess::getSessionKey($this->hash, $user_data['user_id']);
                }
            } else {
                $this->generateOutput('auth_error');
            }
        } elseif ($this->session_key && MobileassistantconnectorAccess::checkSessionKey($this->session_key)) {
            $session_key = $this->session_key;
        }

        $this->generateOutput(array('session_key' => $session_key));
    }

    private function getSortDirection($direction_default)
    {
        return !empty($this->order_by) ? $this->order_by : $direction_default;
    }

    private function getAlwaysAllowedFunctions()
    {
        $allowed_functions_always = array(
            'run_self_test',
            'get_stores',
            'get_currencies',
            'get_store_title',
            'get_orders_statuses',
            'get_carriers'
        );

        return $allowed_functions_always;
    }
}