<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

class MobileassistantconnectorAccess
{
    const HASH_ALGORITHM     = 'sha256';
    const MAX_LIFETIME       = 86400; /* 24 hours */
    const TABLE_SESSION_KEYS = '#__mobassistantconnector_session_keys';
    const TABLE_FAILED_LOGIN = '#__mobassistantconnector_failed_login';
    const TABLE_USERS        = '#__mobassistantconnector_users';

    private static function setDelay($count_attempts)
    {
        if ($count_attempts > 3 && $count_attempts <= 10) {
            sleep(1);
        } elseif ($count_attempts <= 20) {
            sleep(2);
        } elseif ($count_attempts <= 50) {
            sleep(5);
        } else {
            sleep(10);
        }
    }

    private static function generateSessionKey($user_id)
    {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);

        $timestamp = time();
        $query
            ->select('session_key')
            ->from(self::TABLE_SESSION_KEYS)
            ->where($db->quoteName('user_id') . ' = ' . (int)$user_id . ' AND (' .
                $db->quoteName('date_added') . " >= '" .
                JFilterInput::getInstance()->clean(date('Y-m-d H:i:s', ($timestamp - self::MAX_LIFETIME))) . "')");
        $db->setQuery($query);
        $key = $db->loadResult();

        if ($key) {
            return $key;
        }

        $cookieRandom = rand(0,10000);

        $date = date('Y-m-d H:i:s', $timestamp);
        $key = hash(self::HASH_ALGORITHM,$cookieRandom.$timestamp);

        $query = 'INSERT INTO`' . self::TABLE_SESSION_KEYS
            . "` (`session_key`, `user_id`, `date_added`) VALUES ('"
            . $key . "', " . (int)$user_id . ", '" . $date . "') ON DUPLICATE KEY UPDATE `session_key` = '" . $key
            . "', `date_added` = '" . $date . "'";
        $db->setQuery($query);
        $db->execute();
        return $key;
    }

    public static function clearOldData()
    {
        $timestamp       = time();
        $date_clear_prev = JFactory::getConfig()->get('MOBILEASSISTANTCONNECTOR_CL_DATE');
        $date            = date('Y-m-d H:i:s', ($timestamp - self::MAX_LIFETIME));
        $db = JFactory::getDbo();
        $querySessionKeys = $db->getQuery(true);
        $queryFailedLogin = $db->getQuery(true);

        if ($date_clear_prev === false || ($timestamp - (int)$date_clear_prev) > self::MAX_LIFETIME) {
            $querySessionKeys
                ->delete($db->quoteName(self::TABLE_SESSION_KEYS))
                ->where("`date_added` < '". JFilterInput::getInstance()->clean($date)."'");
            $db->setQuery($querySessionKeys);
            $db->execute();
            $queryFailedLogin
                ->delete(self::TABLE_FAILED_LOGIN)
                ->where("`date_added` < '". JFilterInput::getInstance()->clean($date)."'");
            $db->setQuery($queryFailedLogin);
            $db->execute();
            JFactory::getConfig()->set('MOBILEASSISTANTCONNECTOR_CL_DATE', $timestamp);
        }
    }

    public static function getSessionKey($hash, $user_id = false)
    {
        if (!$user_id) {
            $login_data = self::checkAuth($hash);

            if ($login_data) {
                $user_id = (int)$login_data['user_id'];
            }
        }

        if ($user_id) {
            return self::generateSessionKey($user_id);
        }

        self::addFailedAttempt();
        return false;
    }

    public static function checkSessionKey($key, $user_id = false)
    {

        $db = JFactory::getDbo();
        $query = $db->getQuery(true);

        $timestamp = time();
        $where = array(
            "s.`session_key` = '". JFilterInput::getInstance()->clean($key)."'",
            "s.`date_added` > '". JFilterInput::getInstance()->clean(date('Y-m-d H:i:s', ($timestamp - self::MAX_LIFETIME)))."'",
            'u.`status` = 1'
        );

        if ($user_id) {
            $where[] = 'u.`user_id` = '.(int)$user_id;
        }

        $query
            ->select('s.session_key')
            ->from($db->quoteName(self::TABLE_SESSION_KEYS, 's'))
            ->join('LEFT', $db->quoteName(self::TABLE_USERS, 'u') . ' ON (' . $db->quoteName('s.user_id') . ' = ' . $db->quoteName('u.user_id') . ')')
            ->where(implode(' AND ', $where));
        $db->setQuery($query);
        $db_key = $db->loadResult();

        if ($db_key) {
            return true;
        }

        self::addFailedAttempt();
        return false;
    }

    public static function addFailedAttempt()
    {

        $timestamp = time();
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);

        $query
            ->insert($db->quoteName(self::TABLE_FAILED_LOGIN))
            ->columns(implode(',',array('ip', 'date_added')))
            ->values(implode(',' ,array("'" . $_SERVER['REMOTE_ADDR'] . "', '" . date('Y-m-d H:i:s', $timestamp) . "'")));
        $db->setQuery($query);
        $db->execute();

        // Get count of failed attempts for last 24 hours and set delay
        $count_failed_attempts = 'SELECT COUNT(`ip`) FROM `'.self::TABLE_FAILED_LOGIN."` WHERE `ip` = '"
            .JFilterInput::getInstance()->clean($_SERVER['REMOTE_ADDR'])."' AND `date_added` > '"
            .JFilterInput::getInstance()->clean(date('Y-m-d H:i:s', ($timestamp - self::MAX_LIFETIME)))."'";
        $db->setQuery($count_failed_attempts);
        $db->execute();
        $count_failed_attempts = $db->loadAssocList();

        self::setDelay((int)$count_failed_attempts);
    }

    public static function checkAuth($hash, $file_logger = null)
    {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);

        $query
            ->select($db->quoteName(array('user_id', 'username', 'password')))
            ->from(self::TABLE_USERS)
            ->where('status = 1');
        $db->setQuery($query);
        $db->execute();
        $login_data = $db->loadAssocList();

        if ($login_data) {
            $count = count($login_data);

            for ($i = 0; $i < $count; $i++) {
                if (hash(self::HASH_ALGORITHM, $login_data[$i]['username'].$login_data[$i]['password']) == $hash) {
                    return $login_data[$i];
                }
            }
        }

        if ($file_logger) {
            JFactory::getApplication()->enqueueMessage(JText::_("Hash accepted is incorrect"), 'com_mobileassistantconnector');
        }
        return false;
    }


    public static function getAllowedActionsBySessionKey($key)
    {
        $db = JFactory::getDbo();
        $actions = $db->getQuery(true);
        $result = array();
        $actions
            ->select($db->quoteName('u.allowed_actions'))
            ->from($db->quoteName(self::TABLE_SESSION_KEYS, 's'))
            ->join('LEFT', $db->quoteName(self::TABLE_USERS, 'u') . ' ON( ' . $db->quoteName('u.user_id')
            . ' = ' . $db->quoteName('s.user_id') .')')
            ->where($db->quoteName('s.session_key') . " = '" . $key . "'");

        $db->setQuery($actions);
        $db->execute();
        $actions = $db->loadResult();

        if ($actions) {
            $result = explode(';', $actions);
        }

        return $result;
    }

    public static function getAllowedActionsByUserId($user_id)
    {
        $db = JFactory::getDbo();
        $actions = $db->getQuery(true);
        $result = array();

        $actions
            ->select('allowed_actions')
            ->from($db->quoteName('#__mobassistantconnector_users'))
            ->where($db->quoteName('user_id') . ' = ' . (int)$user_id);

        $db->setQuery($actions);
        $actions = $db->loadAssoc();

        if ($actions) {
            $result = explode(';', $actions['allowed_actions']);
        }

        return $result;
    }

    public static function getUserIdBySessionKey($key)
    {

        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query
            ->select('user_id')
            ->from($db->quoteName(self::TABLE_SESSION_KEYS))
            ->where($db->quoteName('session_key') . ' = "' . $key . '"');
        $db->setQuery($query);
        return $db->loadResult();
    }

    public static function getRestrictedActionsToFunctions()
    {
        $restricted_actions_to_functions = array(
            'push_notification_settings_new_order' => array(
                'push_notification_settings',
                'delete_push_config',
            ),
            'push_notification_settings_new_customer' => array(
                'push_notification_settings',
                'delete_push_config',
            ),
            'push_notification_settings_order_statuses' => array(
                'push_notification_settings',
                'delete_push_config',
            ),
            'store_stats' => array(
                'get_store_stats',
                'get_data_graphs',
                'get_status_stats',
            ),
            'products_list' => array(
                'search_products',
                'search_products_ordered',
            ),
            'product_details' => array(
                'get_products_info',
                'get_products_descr',
            ),
            'customers_list' => array(
                'get_customers',
            ),
            'customer_details' => array(
                'get_customers_info',
            ),
            'orders_list' => array(
                'get_orders',
            ),
            'order_details' => array(
                'get_orders_info',
                'get_order_pdf',
            ),
            'update_order_status' => array(
                'set_order_action',
                'change_status',
            ),
            'update_order_tracking_number' => array(
                'set_order_action',
                'update_track_number',
            ),
            'abandoned_carts_list' => array(
                'get_abandoned_carts_list',
            ),
            'abandoned_cart_details' => array(
                'get_abandoned_cart_details',
            ),
        );

        return $restricted_actions_to_functions;
    }
}
