<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

defined('_JEXEC') or die;

if (!class_exists('CurrencyDisplay')) {
    require_once JPATH_ADMINISTRATOR.'/components/com_virtuemart/helpers/currencydisplay.php';
}

require_once JPATH_SITE . '/components/com_mobileassistantconnector/helpers/devices.php';
require_once JPATH_SITE . '/components/com_mobileassistantconnector/helpers/device_and_push_notifications.php';
require_once JPATH_ADMINISTRATOR . '/components/com_virtuemart/helpers/config.php';
require_once JPATH_ADMINISTRATOR . '/components/com_virtuemart/helpers/vmmodel.php';


jimport ('joomla.plugin.plugin');
jimport('joomla.log.log');

class MobileassistantconnectorHelperNotifications extends JHelperContent
{
    public static function onUserCreated($user)
    {
        $devices   = MobileassistantconnectorHelperDevices::getActiveDevices();
        $count_all = count($devices);
        $type      = 'new_customer';

        for ($i = 0; $i < $count_all; $i++) {
            $devices[$i] = self::preparePushSettings($devices[$i]);

            if (
                !empty($devices[$i]['device_id'])
                && $devices[$i]['new_customer'] == 1
                && $devices[$i]['app_connection_id'] > 0
            ) {
                $store_url = self::getStoreUrl();

                if (null == $user['first_name'] && null == $user['last_name']) {
                    $customer_name = $user['username'];
                } else {
                    $customer_name = $user['first_name'] . ' ' . $user['last_name'];
                }

                $fields = array(
                    'registration_ids' => array($devices[$i]['device_id']),
                    'data' => array(
                        'message' => array(
                            'push_notif_type' => $type,
                            'email' => $user['email'],
                            'customer_name' => $customer_name,
                            'customer_id' => $user['id'],
                            'store_url' => $store_url,
                            'app_connection_id' => $devices[$i]['app_connection_id']
                        )
                    )
                );

                $fields = json_encode($fields);
                $response = self::sendPushMessage($fields);
                self::proceedGoogleResponse($response, $devices[$i]['device_id']);
            }
        }
    }

    public static function onOrderCreated($order)
    {
        $devices   = MobileassistantconnectorHelperDevices::getActiveDevices();
        $count_all = count($devices);

        if ($count_all < 1) {
            return;
        }

        $type = 'new_order';
        $order_details = $order['details']['BT'];
        $total = $order_details->order_total;
        $status = VmText::_($order_details->order_status);
        $GLOBALS['hookNewOrder'] = 1;

        for ($i = 0; $i < $count_all; $i++) {
            $devices[$i] = self::preparePushSettings($devices[$i]);

            if (
                !empty($devices[$i]['device_id'])
                && $devices[$i]['new_order'] == 1
                && $devices[$i]['app_connection_id'] > 0
            ) {
                // Send data to Google
                self::sendData(
                    $devices[$i]['app_connection_id'],
                    $devices[$i]['device_id'],
                    $type,
                    $order_details->email,
                    $order_details->first_name . ' ' . $order_details->last_name,
                    $order_details->virtuemart_order_id,
                    self::getTotal($devices[$i]['currency_code'], $order_details->order_currency, $total),
                    $status
                );
            }
        }
    }

    private static function getStoreUrl()
    {
        $store_url = JUri::root();
        $store_url  = str_replace('http://', '', $store_url);
        $store_url  = str_replace('https://', '', $store_url);
        preg_replace('/\/*$/i', '', $store_url);

        return $store_url;
    }

    private static function preparePushSettings($data)
    {
        if (!is_array($data)) {
            $data = array();
        }

        $currency_model = VmModel::getModel('currency');
        $displayCurrency = $currency_model->getCurrency($data['currency_code']);
        $currency = $displayCurrency->virtuemart_currency_id;
        $default_currency = CurrencyDisplay::getInstance()->getId();

        $data['device_id'] = (!isset($data['device_id']) || is_null($data['device_id']))
            ? ''
            : (string)$data['device_id'];
        $data['app_connection_id'] = (!isset($data['app_connection_id']) || (int)$data['app_connection_id'] < 1)
            ? 0
            : (int)$data['app_connection_id'];
        $data['currency_code'] = (!isset($data['currency_code']) || (int)$data['currency_code'] < 1)
            ? $default_currency
            : $currency;
        $data['id_shop'] = (!isset($data['app_connection_id']) || (int)$data['id_shop'] < 1)
            ? '-1'
            : (int)$data['id_shop'];
        $data['status'] = !isset($data['status'])
            ? 1
            : (int)$data['status'];
        $data['new_order'] = !isset($data['new_order'])
            ? 1
            : (int)$data['new_order'];
        $data['new_customer'] = !isset($data['new_customer'])
            ? 1
            : (int)$data['new_customer'];
        $data['order_statuses'] = !isset($data['order_statuses'])
            ? ''
            : (string)$data['order_statuses'];

        return $data;
    }

    private static function sendPushMessage($message_content)
    {
        $api_key = JComponentHelper::getParams('com_mobileassistantconnector')->get('api_key');

        if (!$api_key) {
            return false;
        }

        $headers = array("Authorization: key=$api_key", 'Content-Type: application/json');
        $result = false;

        if (is_callable('curl_init')) {
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, 'https://android.googleapis.com/gcm/send');
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $message_content);
            $result = curl_exec($ch);

            if (curl_errno($ch)) {
                JLog::add(JText::_(
                    "Push message error while sending CURL request: {$result}"),
                    JLog::WARNING,
                    'com_mobileassistantconnector'
                );
            }
            curl_close($ch);
        }

        return $result;
    }

    private static function proceedGoogleResponse($response, $device_registration_id)
    {
        $db = JFactory::getDbo();
        $json = array();

        if ($response && strpos($response, '{') === 0) {
            $json = json_decode($response, true);

            if (!is_array($json)) {
                $json = array();
            }
        }

        if (empty($json) || !isset($json['results'])) {
            return;
        }

        $results = is_array($json['results']) ? $json['results'] : array();

        foreach ($results as $result) {
            if (
                isset($result['registration_id']) && isset($json['canonical_ids']) && (int)$json['canonical_ids'] > 0 ||
                isset($result['error']) &&
                ($result['error'] == 'NotRegistered' || $result['error'] == 'InvalidRegistration')
            ) {
                if (
                    isset($result['registration_id'])
                    && isset($json['canonical_ids'])
                    && (int)$json['canonical_ids'] > 0
                ) {
                    $query = $db->getQuery(true);
                    $query
                        ->select('pn.id')
                        ->from($db->quoteName('#__mobassistantconnector_push_notifications', 'pn'))
                        ->leftJoin('#__mobassistantconnector_push_notifications AS pn1 
                            ON pn1.app_connection_id = pn.app_connection_id AND pn1.user_id = pn.user_id')
                        ->where('pn.device_id = ' . $device_registration_id . ' AND pn1.device_id = ' . $result['registration_id']);
                    $db->setQuery($query);
                    $push_ids = $db->loadAssoc();

                    // Delete old records if new push settings are stored
                    $query_delete = $db->getQuery(true);
                    $query_delete
                        ->delete($db->quoteName('#__mobassistantconnector_push_notifications'))
                        ->where('id = IN(' . explode(',',$push_ids) .')');
                    $db->setQuery($query_delete);
                    $db->execute();

                    // Replace old registration ids with new
                    $query_update = $db->getQuery(true);
                    $query_update
                        ->update($db->quoteName('#__mobassistantconnector_push_notifications'))
                        ->set('device_id = ' . $result['registration_id'] . ' AND device_id = ' . $device_registration_id);
                    $db->setQuery($query_update);
                    $db->execute();

                } else {
                    $query_delete = $db->getQuery(true);
                    $query_delete
                        ->delete($db->quoteName('#__mobassistantconnector_push_notifications'))
                        ->where('device_id = ' . $device_registration_id);

                    MobileassistantconnectorHelperDevice_and_push_notifications::deleteEmptyDevices();
                    MobileassistantconnectorHelperDevice_and_push_notifications::deleteEmptyAccounts();
                    JLog::add(JText::_("Google error response: {$response}"), JLog::WARNING, 'com_mobileassistantconnector');
                }
            }
        }
    }

    private static function getTotal($device_currency_code, $order_currency_id, $total)
    {
        $currency = CurrencyDisplay::getInstance($device_currency_code);

        if ($device_currency_code != $order_currency_id) {
            $new_currency_total = $currency->convertCurrencyTo($device_currency_code, $total, false);
        } else {
            $new_currency_total = $total;
        }

        $display_currency = $currency->getFormattedCurrency($new_currency_total);
        return $display_currency;
    }

    private static function sendData(
        $app_connection_id,
        $device_id,
        $type,
        $customer_email,
        $customer_name,
        $id_order,
        $total,
        $status_code = false
    ) {
        $store_url = self::getStoreUrl();
        $fields = array(
            'registration_ids' => array($device_id),
            'data' => array(
                'message' => array(
                    'push_notif_type'   => $type,
                    'email'             => $customer_email,
                    'customer_name'     => $customer_name,
                    'order_id'          => $id_order,
                    'total'             => $total,
                    'store_url'         => $store_url,
                    'app_connection_id' => $app_connection_id
                )
            ),
        );

        if ($status_code) {
            $fields['data']['message']['new_status_code'] = $status_code;
        }

        $fields = json_encode($fields);
        $response = self::sendPushMessage($fields);
        self::proceedGoogleResponse($response, $device_id);
    }
}